@@ -1,4 +1,4 @@
-; ACL2 Version 3.0 -- A Computational Logic for Applicative Common Lisp
+; ACL2 Version 3.1 -- A Computational Logic for Applicative Common Lisp
 ; Copyright (C) 2006  University of Texas at Austin
 
 ; This version of ACL2 is a descendent of ACL2 Version 1.9, Copyright
@@ -51,6 +51,14 @@
 ;                            PROCLAIMING
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
+(defmacro defun-one-output (&rest args)
+
+; Use this for raw Lisp functions that are known to return a single value in
+; raw Lisp, since make-defun-declare-form uses that assumption to make an
+; appropriate declaration.
+
+  (cons 'defun args))
+
 (defun macroexpand-till (form sym)
 
 ; In order to find the THEs that we want to find to do automatic
@@ -107,6 +115,26 @@
         "Unexpected type for convert-to-integer-type ~s"
         typ))))
 
+#+(or akcl openmcl)
+(defconstant *lisp-fns-with-mv-output-type*
+
+; We have used the following form in GCL to compute the list of functions that
+; return multiple values.  Presumably this is the correct list for the main
+; Lisp package in other Common Lisp implementations as well.
+
+; (let (ans)
+;   (do-symbols (sym (find-package "LISP"))
+;               (let ((tp (get sym 'compiler::return-type)))
+;                 (when (and (consp tp) (eq (car tp) 'values))
+;                   (setq ans (cons sym ans)))))
+;   ans)
+
+  '(INTERN DECODE-FLOAT READ-LINE MACROEXPAND ROUND GETHASH CEILING
+           MACROEXPAND-1 INTEGER-DECODE-FLOAT TRUNCATE FIND-SYMBOL FLOOR))
+
+#+(or akcl openmcl)
+(defvar *acl2-output-type-abort* nil)
+
 #+(or akcl openmcl)
 (defun max-output-type-for-declare-form (type1 type2)
 
@@ -117,24 +145,36 @@
   (cond
    ((equal type1 type2)
     type1)
-   ((or (null type1)
-        (null type2))
-    nil)
-   ((or (eq type1 'integer)
-        (eq type2 'integer)
-        (and (consp type1)
-             (eq (car type1) 'integer)
-             (or (and (consp (cdr type1))
-                      (null (cddr type1)))
-                 (member '* (cdr type1) :test 'eq)))
-        (and (consp type2)
-             (eq (car type2) 'integer)
-             (or (and (consp (cdr type2))
-                      (null (cddr type2)))
-                 (member '* (cdr type2) :test 'eq))))
+   ((or (eq type1 '*)
+        (eq type2 '*))
+    '*)
+   #+acl2-mv-as-values
+   ((not (equal (and (consp type1)
+                     (eq (car type1) 'values))
+                (and (consp type2)
+                     (eq (car type2) 'values))))
+    '*)
+   ((and (or (eq type1 'integer)
+             (and (consp type1)
+                  (eq (car type1) 'integer)
+                  (or (null (cddr type1))
+                      (member '* (cdr type1) :test 'eq))))
+         (or (eq type2 'integer)
+             (and (consp type2)
+                  (eq (car type2) 'integer)
+                  (or (null (cddr type2))
+                      (member '* (cdr type2) :test 'eq)))))
     'integer)
-   ((or (atom type1) (atom type2))
-    nil)
+   ((or (atom type1) (atom type2)) ; so, type is t since neither is *
+    t)
+   ((cdr (last type1)) ; (not (true-listp type1))
+    (error
+     "Non-atom, non-true-listp type for max-output-type-for-declare-form: ~s"
+     type1))
+   ((cdr (last type2)) ; (not (true-listp type2))
+    (error
+     "Non-atom, non-true-listp type for max-output-type-for-declare-form: ~s"
+     type2))
    (t (let ((sym1 (car type1))
             (sym2 (car type2)))
         (cond
@@ -148,6 +188,7 @@
              (list 'integer
                    (min  (cadr type1)  (cadr type2))
                    (max (caddr type1) (caddr type2))))
+            #+acl2-mv-as-values
             (values
              (cons 'values (max-output-type-for-declare-form-lst (cdr type1)
                                                                  (cdr type2))))
@@ -155,9 +196,9 @@
              (error
               "Unexpected type for max-output-type-for-declare-form: ~s"
               type1))))
-         ((or (eq sym1 'values)
-              (eq sym2 'values))
-          nil)
+         #+acl2-mv-as-values
+         ((or (eq sym1 'values) (eq sym2 'values)) ; but not both
+          '*)
          (t (let* ((pair1 (convert-type-to-integer-pair type1))
                    (pair2 (convert-type-to-integer-pair type2))
                    (lower1 (car pair1))
@@ -176,16 +217,26 @@
 
 #+(and (or akcl openmcl) acl2-mv-as-values)
 (defun max-output-type-for-declare-form-lst (type-list1 type-list2)
-  (cond ((or (atom type-list1) (atom type-list2))
+
+; Type-list1 and type-list2 are known to be true lists (null-terminated
+; lists).
+
+  (cond ((or (null type-list1) (null type-list2))
          (cond
-          ((and (atom type-list1) (atom type-list2))
+          ((and (null type-list1) (null type-list2))
            nil)
+          ((and *acl2-output-type-abort*
+                (or (atom type-list1) (atom type-list2)))
+           (cons '*
+                 (max-output-type-for-declare-form-lst
+                  (cdr type-list1) (cdr type-list2))))
           (t
            (error "Implementation error:~%~
                    max-output-type-for-declare-form-lst called on lists of~%~
                    different length:~%~
                    ~s~%  ~s~%~
-                   Please contact the ACL2 implementors."))))
+                   Please contact the ACL2 implementors."
+                  type-list1 type-list2))))
         (t (cons (max-output-type-for-declare-form
                   (car type-list1) (car type-list2))
                  (max-output-type-for-declare-form-lst
@@ -201,40 +252,98 @@
    ((integerp form)
     `(integer ,form ,form))
    ((atom form)
-    nil)
+    t)
    ((eq (car form) 'the)
     (let ((typ (cadr form)))
-      (cond ((eq typ 'integer)
+      (cond ((member typ '(integer fixnum) :test 'eq)
              typ)
             ((and (consp typ)
                   (member (car typ)
-                          '(integer signed-byte unsigned-byte values)))
-             typ))))
+                          '(integer signed-byte unsigned-byte
+                                    #+acl2-mv-as-values
+                                    values)
+                          :test 'eq))
+             typ)
+            (t t))))
    ((eq (car form) 'if)
-    (let ((type1 (output-type-for-declare-form-rec (caddr form))))
-      (and type1 ; optimization
-           (max-output-type-for-declare-form
-            type1
-            (output-type-for-declare-form-rec (cadddr form))))))
+    (cond
+     ((eq (cadr form) t) ; as generated for final cond branch in OpenMCL
+      (output-type-for-declare-form-rec (caddr form)))
+     ((eq (cadr form) nil) ; perhaps not necessary
+      (output-type-for-declare-form-rec (cadddr form)))
+     (t (let ((type1 (output-type-for-declare-form-rec (caddr form))))
+          (if (eq type1 '*) ; optimization
+              '*
+            (max-output-type-for-declare-form
+             type1
+             (output-type-for-declare-form-rec (cadddr form))))))))
    #+acl2-mv-as-values
    ((eq (car form) 'values)
-    (cons 'values (output-type-for-declare-form-rec-list (cdr form))))
+    (cond ((null (cddr form)) ; e.g., from (cond (a))
+           (output-type-for-declare-form-rec (cadr form)))
+          (t
+           (cons 'values (output-type-for-declare-form-rec-list (cdr form))))))
+   ((member (car form) '(let let*) :test 'eq)
+    (cond ((cddr form)
+           (output-type-for-declare-form-rec (car (last form))))
+          (t t)))
+   #+acl2-mv-as-values
+   ((eq (car form) 'multiple-value-bind)
+    (cond ((cdddr form)
+           (output-type-for-declare-form-rec (car (last form))))
+          (t t)))
+   ((member (car form) '(time progn) :test 'eq)
+    (output-type-for-declare-form-rec (car (last form))))
+   ((member (car form)
+            '(tagbody ; e.g., ld-fn
+              throw-raw-ev-fncall ; e.g., from defchoose
+              )
+            :test 'eq)
+    (setq *acl2-output-type-abort* '*))
    (t (multiple-value-bind
        (new-form flg)
        (macroexpand-1 form)
-       (cond ((null flg) nil)
+       (cond ((null flg)
+              (cond ((atom form) t)
+                    ((eq (car form) 'multiple-value-prog1)
+                     (and (consp (cdr form))
+                          (output-type-for-declare-form-rec (cadr form))))
+; Note: We don't expect multiple-value-setq to show up in ACL2.
+                    ((and (consp (car form))
+                          (eq (caar form) 'lambda))
+                     (output-type-for-declare-form-rec (caddr (car form))))
+                    ((or (not (symbolp (car form))) ; should always be false
+                         (member (car form)
+                                 *lisp-fns-with-mv-output-type*
+                                 :test 'eq))
+                     '*)
+                    #-acl2-mv-as-values
+                    (t t)
+                    #+acl2-mv-as-values
+                    (t (let ((x (and (eval '(f-get-global 'current-acl2-world
+                                                          *the-live-state*))
+                                     (funcall 'get-stobjs-out-for-declare-form
+                                              (car form)))))
+                         (cond ((consp (cdr x))
+                                (cons 'values
+                                      (make-list (length x)
+                                                 :initial-element
+                                                 t)))
+                               (t t))))))
              (t (output-type-for-declare-form-rec new-form)))))))
        
 #+(and (or akcl openmcl) acl2-mv-as-values)
 (defun output-type-for-declare-form-rec-list (forms)
   (cond ((atom forms)
          nil)
-        (t (cons (or (output-type-for-declare-form-rec (car forms))
-                     t)
+        (t (cons (let ((tp (output-type-for-declare-form-rec (car forms))))
+                   (if (member tp '(nil *) :test 'eq)
+                       t
+                     tp))
                  (output-type-for-declare-form-rec-list (cdr forms))))))
 
 #+(or akcl openmcl)
-(defun output-types-for-declare-form (fn form)
+(defun output-type-for-declare-form (fn form)
 
 ; We return a list of output types, one per value.  So if #-acl2-mv-as-values,
 ; then we always return a list of length one.
@@ -242,63 +351,69 @@
   #-acl2-mv-as-values
   (declare (ignore fn))
   #-acl2-mv-as-values
-  (list (or (output-type-for-declare-form-rec form)
-            t))
+  (list 'values (output-type-for-declare-form-rec form))
   #+acl2-mv-as-values
-  (let ((result (output-type-for-declare-form-rec form))
-        (stobjs-out
-         (eval `(let ((wrld (w *the-live-state*)))
-                  (and wrld ; cheap check to avoid any early proclaim-files
-                       (getprop ',fn 'stobjs-out nil 'current-acl2-world
-                                wrld))))))
+  (let* ((*acl2-output-type-abort* nil) ; protect for call on next line
+         (result (output-type-for-declare-form-rec form))
+         (stobjs-out (and (eval '(f-get-global 'current-acl2-world
+                                               *the-live-state*))
+                          (funcall 'get-stobjs-out-for-declare-form fn))))
     (when (and stobjs-out
-               result
-               (or (and (consp result)
-                        (eq (car result) 'values)
-                        (null (cdr stobjs-out)))
-                   (and (or (atom result)
-                            (not (eq (car result) 'values)))
-                        (cdr stobjs-out))))
-      (error "Implementation error in output-types-for-declare-form:~%~
+               (not (eq (and (consp result)
+                             (eq (car result) 'values))
+                        (consp (cdr stobjs-out))))
+               (not *acl2-output-type-abort*))
+      (error "Implementation error in ~s:~%~
               stobjs-out and result don't mesh.~%~
               Stobjs-out = ~s~%~
               Result = ~s~%~
               Please contact the ACL2 implementors."
+             (list 'output-type-for-declare-form fn '|defun...|)
              stobjs-out result))
     (cond
      ((and (consp result)
            (eq (car result) 'values))
-      (cdr result))
-     (result
-      (list result))
-     (stobjs-out
-      (make-list (length stobjs-out) :initial-element t))
-; We allow this function to be called even on functions for which there is null
-; stobjs-out (hence, functions defined only in raw Lisp), but if no output type
-; is computed then with stobjs-out equal to nil we cannot compute an output
-; type, so we return nil.
-     )))
+      result)
+     (*acl2-output-type-abort*
+      '*)
+     (t
+      (list 'values result)))))
 
-(defun make-defun-declare-form (fn form)
+(defun make-defun-declare-form (fn form
+                                   &optional
+                                   (output-type nil output-type-p))
 
 ; See the comment in proclaim-file for why we don't proclaim in more lisps.
 
-  #-(or akcl openmcl) (declare (ignore fn form))
+  #-(or akcl openmcl) (declare (ignore fn form output-type output-type-p))
   #-(or akcl openmcl) nil
 
   #+(or akcl openmcl)
-  (let* ((output-types (output-types-for-declare-form fn (car (last form)))))
-    (and output-types ; only need to check in the case #+acl2-mv-as-values
-         (let ((formals (caddr form)))
-           (cond
-            ((null (intersection formals lambda-list-keywords))
-             `(declaim (ftype (function
-                               ,(arg-declarations
-                                 formals
-                                 (collect-types (cdddr form)))
-                               ,@output-types)
-                              ,(cadr form))))
-            (t nil))))))
+  (let* ((output-type
+          (if output-type-p
+              output-type
+            (output-type-for-declare-form fn (car (last form))))))
+    (let ((formals (caddr form)))
+      (cond
+       ((null (intersection formals lambda-list-keywords
+                            :test 'eq))
+        `(declaim (ftype (function
+                          ,(arg-declarations
+                            formals
+                            (collect-types (cdddr form)))
+                          ,output-type)
+
+; WARNING: Do not replace (cadr form) by fn below.  These can differ!  Fn is
+; passed to output-type-for-declare-form in order to get its 'stobjs-out, but
+; (cadr form) can be the *1* function for fn.  The mistaken placement of fn
+; below caused a factor of 4 slowdown in GCL in the first lemma5 in
+; books/unicode/utf8-decode.lisp, because the proclaim for function
+; utf8-combine4-guard was overwritten by a subsequent weaker proclaimed type
+; that was supposed to be generated for the *1* function, but instead was
+; generated for utf8-combine4-guard.
+
+                         ,(cadr form))))
+       (t nil)))))
 
 #+(or akcl openmcl)
 (defun make-defconst-declare-form (form)
@@ -328,7 +443,14 @@
 ; system.
 
          (template (funcall 'defstobj-template name args))
-         (raw-defs (funcall 'defstobj-raw-defs name template)))
+         (raw-defs (funcall 'defstobj-raw-defs name template
+
+; We do not want to rely on having the world available here, so we pass in nil
+; for the final argument of defstobj-raw-defs.  The only effect of using nil
+; instead of a world in such a context is that additional checking by
+; translate-declaration-to-guard is missing.
+
+                            nil)))
     (cons 'progn
           (mapcar (function
                    (lambda (def) (if (member (symbol-value
@@ -367,8 +489,15 @@
                 (dolist (x (cdr form))
                         (proclaim-form x stream))
                 nil)
-               ((defun)
-                (eval-or-print (make-defun-declare-form (cadr form) form) stream)
+               ((defun defund)
+                (eval-or-print (make-defun-declare-form (cadr form) form)
+                               stream)
+                nil)
+               (defun-one-output
+                (eval-or-print (make-defun-declare-form (cadr form)
+                                                        form
+                                                        '(values t))
+                               stream)
                 nil)
                (otherwise nil)))
         (t nil)))
@@ -640,7 +769,7 @@ notation causes an error and (b) the use of ,. is not permitted."
 
 (defun *1*-symbol (x)
   (or (get x **1*-symbol-key*)
-      (setf **1*-symbol-key*
+      (setf (get x **1*-symbol-key*)
             (intern (symbol-name x)
                     (find-package (concatenate 'string
                                                *1*-package-prefix*
@@ -672,7 +801,9 @@ notation causes an error and (b) the use of ,. is not permitted."
                   (list 'ev-fncall-guard-er
                         'funny-fn
                         ,(cons 'list 'funny-args)
-                        (guard 'funny-fn nil (w *the-live-state*))
+                        (untranslate* (guard 'funny-fn nil (w *the-live-state*))
+                                      t
+                                      (w *the-live-state*))
 
 ; We assume that the primitives don't take state.
 
@@ -686,22 +817,24 @@ notation causes an error and (b) the use of ,. is not permitted."
 
 ; The following is first used in acl2-init.lisp, so we define it here.
 
-(defun getenv$ (string)
+(defun getenv$-raw (string)
 
 ; The following either returns the value of the given environment variable or
 ; returns nil (in lisps where we do not yet know how to get that value).
 
-  (or #+cmu
-      (and (boundp ext::*environment-list*)
-           (cdr (assoc-eq (intern string :keyword)
-                          ext::*environment-list*)))
-      #+(or gcl allegro lispworks openmcl sbcl clisp)
-      (let ((fn
-             #+gcl       'si::getenv
-             #+allegro   'sys::getenv
-             #+lispworks 'cl::getenv
-             #+openmcl   'ccl::getenv
-             #+sbcl      'sb-ext::posix-getenv
-             #+clisp     'ext:getenv))
-        (and (fboundp fn)
-             (funcall fn string)))))
+; WARNING: Keep this in sync with the #-acl2-loop-only definition of setenv$.
+
+  #+cmu
+  (and (boundp ext::*environment-list*)
+       (cdr (assoc-eq (intern string :keyword)
+                      ext::*environment-list*)))
+  #+(or gcl allegro lispworks openmcl sbcl clisp)
+  (let ((fn
+         #+gcl       'si::getenv
+         #+allegro   'sys::getenv
+         #+lispworks 'cl::getenv
+         #+openmcl   'ccl::getenv
+         #+sbcl      'sb-ext::posix-getenv
+         #+clisp     'ext:getenv))
+    (and (fboundp fn)
+         (funcall fn string))))
