@@ -1,4 +1,4 @@
-; ACL2 Version 3.0 -- A Computational Logic for Applicative Common Lisp
+; ACL2 Version 3.1 -- A Computational Logic for Applicative Common Lisp
 ; Copyright (C) 2006  University of Texas at Austin
 
 ; This version of ACL2 is a descendent of ACL2 Version 1.9, Copyright
@@ -174,33 +174,7 @@
 
          (setq source-directory
                (subseq source-directory 0 (1- (length source-directory))))))
-  (let ((Tantallon-projects-acl2 "/v/hank/v92/acl2")
-        (Ravenscraig-projects-acl2 "/v/hank/v40/acl2"))
-
-; On Tantallon, the /projects/acl2/ directory's truename is
-; "/v/hank/v92/acl2/" but on Ravenscraig it is "/v/hank/v40/acl2/".
-; The Tantallon name is built into our Makefile.  Therefore, if this
-; function is asked to use "/v/hank/v92/acl2/..." as the source
-; directory, but we are running on Ravenscraig, we will use
-; "/v/hank/v40/..." instead.
-
-    (cond ((and (<= (length Tantallon-projects-acl2)
-                    (length source-directory))
-                (equal (subseq source-directory
-                               0
-                               (length Tantallon-projects-acl2))
-                       Tantallon-projects-acl2)
-                (equal (subseq (namestring (truename "/projects/acl2/"))
-                               0
-                               (length Ravenscraig-projects-acl2))
-                       Ravenscraig-projects-acl2))
-           (setq source-directory
-                 (format nil "~a~a"
-                         Ravenscraig-projects-acl2
-                         (subseq source-directory
-                                 (length Tantallon-projects-acl2)))))))
-
-  (cond ((not (equal (format nil "~a/" source-directory)
+  (cond ((not (equal (namestring (truename (format nil "~a/" source-directory)))
                      (namestring (truename ""))))
          (error "We expected to be in the directory~%~s~%~
                  but instead are apparently in the directory~%~s .~%~
@@ -442,7 +416,7 @@
 (defmacro write-exec-file (stream string &rest args)
   `(progn (format ,stream
                   "#!/bin/sh~%~a~%"
-                  (let ((books-dir (getenv$ "ACL2_SYSTEM_BOOKS")))
+                  (let ((books-dir (getenv$-raw "ACL2_SYSTEM_BOOKS")))
                     (if books-dir
                         (format nil "export ACL2_SYSTEM_BOOKS=~a" books-dir)
                       "")))
@@ -1130,7 +1104,10 @@
          (core-name (concatenate 'string
                                  current-dir
                                  core-name
-                                 ".dppccl")))
+                                 "."
+                                 (pathname-name ; Gary Byers suggestion
+                                  (car ccl::*command-line-argument-list*))
+                                 )))
     (when (probe-file sysout-name)
 
 ; At one point we supplied :if-exists :overwrite in the first argument to
