@@ -1,4 +1,4 @@
-; ACL2 Version 3.0 -- A Computational Logic for Applicative Common Lisp
+; ACL2 Version 3.1 -- A Computational Logic for Applicative Common Lisp
 ; Copyright (C) 2006  University of Texas at Austin
 
 ; This version of ACL2 is a descendent of ACL2 Version 1.9, Copyright
@@ -449,7 +449,7 @@
    (setq acl2::*copy-of-acl2-version*
 ;  Keep this in sync with the value of acl2-version in *initial-global-table*.
          (concatenate 'string
-                      "ACL2 Version 3.0"
+                      "ACL2 Version 3.1"
                       #+non-standard-analysis
                       "(r)"
                       #+(and mcl (not openmcl))
@@ -1536,7 +1536,7 @@ Newline
         (catch *quit-tag*
           (setq - (locally (declare (notinline read))
                            (dbl-read *debug-io* nil *top-eof*)))
-          (when (eq - *top-eof*) (bye -1))
+          (when (eq - *top-eof*) (lisp::bye -1))
           (let* ( break-command
                  (values
                   (multiple-value-list
