@@ -1206,7 +1206,8 @@ Robert
 ;;; A couple of neat rules taken from the RTL books.  These are
 ;;; due to Eric Smith.
 
-(defthm integerp-+-reduce-leading-constant
+;renamed to avoid conflict with rtl/rel5
+(defthm integerp-+-reduce-leading-rational-constant
   (implies (and (syntaxp (rational-constant-p c))
                 (syntaxp (or (<= 1 (unquote c))
                              (< (unquote c) 0))))
