@@ -102,8 +102,7 @@
 
 (in-theory (disable insert-0))
 
-(theory-invariant (not (member-equal '(:definition insert-0)
-                                     theory))
+(theory-invariant (not (active-runep '(:definition insert-0)))
                   :error nil)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
