@@ -123,10 +123,8 @@
 (prefer-positive-addends)
 
 (theory-invariant 
- (not (and (member-equal '(:rewrite |(expt x (+ m n))|)
-                         theory)
-           (member-equal '(:rewrite normalize-factors-gather-exponents)
-                         theory)))
+ (not (and (active-runep '(:rewrite |(expt x (+ m n))|))
+           (active-runep '(:rewrite normalize-factors-gather-exponents))))
  :error nil)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
