@@ -107,16 +107,16 @@
 ;;  (integerp (floor x y)))
 
 (defthm integerp-mod
-  (implies (and (integerp x)
-		(integerp y))
-	   (integerp (mod x y)))
-  :rule-classes :type-prescription)
+    (implies (and (integerp m)
+		  (integerp n))
+	     (integerp (mod m n)))
+  :rule-classes (:rewrite :type-prescription))
 
 (defthm rationalp-mod
-  (implies (and (rationalp x)
-		(rationalp y))
-	   (rationalp (mod x y)))
-  :rule-classes :type-prescription)
+  (implies (rationalp x)
+           (rationalp (mod x y)))
+  :hints (("Goal" :cases ((rationalp y))))
+  :rule-classes (:rewrite :type-prescription))
 
 (defthm floor-completion
   (implies (or (not (acl2-numberp x))
@@ -166,6 +166,20 @@
   :hints (("Goal" :in-theory (disable floor)))
   :rule-classes ((:rewrite)
 		 (:elim)))
+
+; The following is an improvement of the above, but during development after
+; Version_3.0.1 release, it was found to ruin the proof of
+; wp-zcoef-loop-invariant in
+; books/workshops/2004/legato/support/generic-theory-loop-invariant-mult.lisp.
+; Rather than investigate, we will just keep the rule above, which isn't too
+; bad.
+
+; (defthm floor-mod-elim
+;   (implies (force (acl2-numberp x))
+;            (equal (+ (mod x y) (* y (floor x y))) x))
+;   :rule-classes (:rewrite :elim)
+;   :hints (("Goal" :in-theory (enable mod))))
+
 
 ;;-----------------------------------------------------------------
 ;;
@@ -733,7 +747,8 @@
 		       (- (floor x (- y)))
 		     (+ (- (floor x (- y))) -1)))))
 
-(defthm mod-minus
+;renamed to avoid conflict with rtl/rel5
+(defthm mod-neg
   (implies (and (rationalp-guard x y)
                 (syntaxp (negative-addends-p x)))
 	   (equal (mod x y)
@@ -1078,4 +1093,4 @@
 (defthm mod-x-1
     (implies (integerp x)
              (equal (mod x 1)
-                    0)))
\ No newline at end of file
+                    0)))
