@@ -360,8 +360,8 @@ Then:
 (in-theory (disable Uniqueness-of-*-inverses))
 
 (theory-invariant
- (not (and (member-equal '(:rewrite Uniqueness-of-*-inverses) theory)
-           (member-equal '(:rewrite equal-/) theory))))
+ (not (and (active-runep '(:rewrite Uniqueness-of-*-inverses))
+           (active-runep '(:rewrite equal-/)))))
 
 (encapsulate
  ()
