@@ -255,6 +255,89 @@
 	 (not (assoc j l)))
     (not (assoc j (compress11 name l i n default))))))
 
+; Start events added by Matt K. for Version 3.1, 7/1/06, to support proof of
+; compress1-assoc-property-0 in light of addition of reverse-sorted and
+; unsorted ACL2 arrays.
+
+(local
+ (defthm assoc-revappend-1
+   (implies (not (member key (strip-cars alist1)))
+            (equal (assoc key (revappend alist1 alist2))
+                   (assoc key alist2)))))
+
+(local
+ (defthm assoc-revappend
+   (implies (and (force (no-duplicatesp (strip-cars alist1)))
+                 (force (alistp alist1)))
+            (equal (assoc key (revappend alist1 alist2))
+                   (or (assoc key alist1)
+                       (assoc key alist2))))))
+
+(local
+ (defun ordered-alistp (x)
+   (cond
+    ((atom x)
+     (null x))
+    ((atom (cdr x))
+     (and (consp (car x))
+          (rationalp (caar x))
+          (null (cdr x))))
+    (t (and (consp (car x))
+            (rationalp (caar x))
+            (< (caar x) (caadr x))
+            (ordered-alistp (cdr x)))))))
+
+(local
+ (defthm no-duplicatesp-strip-cars-ordered-alistp-1
+   (implies (and (< key (caar x))
+                 (ordered-alistp x))
+            (not (member key
+                         (strip-cars x))))))
+
+(local
+ (defthm no-duplicatesp-strip-cars-ordered-alistp
+   (implies (ordered-alistp x)
+            (no-duplicatesp (strip-cars x)))))
+
+(local
+ (defthm consp-assoc-rewrite
+   (implies (and key
+                 (assoc key alist))
+            (consp (assoc key alist)))))
+
+(local
+ (defthm car-assoc
+   (implies (assoc key alist)
+            (equal (car (assoc key alist))
+                   key))))
+
+(local
+ (defthm <-caar-compress11
+   (implies (and (< i j)
+                 (consp (compress11 name l j n default)))
+            (< i (caar (compress11 name l j n default))))))
+
+(local
+ (defthm ordered-alistp-compress11
+   (implies (and (integerp i)
+                 (integerp n))
+            (ordered-alistp (compress11 name l i n default)))))
+
+(local
+ (defthm not-member-strip-cars-compress11
+   (implies
+    (< i j)
+    (not (member i
+                 (strip-cars (compress11 name l j n default)))))))
+
+(local
+ (defthm no-duplicatesp-strip-cars-compress11
+   (no-duplicatesp (strip-cars (compress11 name l i n default)))))
+
+; End events added by Matt K. for Version 3.1, 7/1/06, to support proof of
+; compress1-assoc-property-0 in light of addition of reverse-sorted and
+; unsorted ACL2 arrays.
+
 (local
  (defthm compress1-assoc-property-0
   (implies
@@ -306,6 +389,22 @@
 ;;;
 ;;;****************************************************************************
 
+; The following two events were added by Matt K. for Version 3.1, 7/1/06, to
+; support proof of compress1-assoc-property-0 in light of addition of
+; reverse-sorted and unsorted ACL2 arrays.
+
+(local
+ (defthm alistp-revappend
+   (implies (alistp x)
+            (equal (alistp (revappend x y))
+                   (alistp y)))))
+
+(local
+ (defthm bounded-integer-alistp-revappend
+   (implies (bounded-integer-alistp x n) 
+            (equal (bounded-integer-alistp (revappend x y) n)
+                   (bounded-integer-alistp y n)))))
+
 (defthm array1p-compress1
   (implies
    (array1p name l)
