@@ -95,7 +95,8 @@
                      (null (cdr mtree))))))))
 
 (defun _memtree-fix (mtree depth)
-  (declare (xargs :guard (and (natp depth)
+  (declare (ignorable depth) ; added by Matt K. after 3.0.1 to suppress compiler warning
+           (xargs :guard (and (natp depth)
                               (_memtree-p mtree depth))))
   (if (mbt (_memtree-p mtree depth))
       mtree
@@ -130,7 +131,8 @@
        (< addr (expt 2 (nfix depth)))))
 
 (defun _address-fix (addr depth)
-  (declare (xargs :guard (and (natp depth)
+  (declare (ignorable depth) ; added by Matt K. after 3.0.1 to suppress compiler warning
+           (xargs :guard (and (natp depth)
                               (_address-p addr depth))))
   (if (mbt (_address-p addr depth))
       addr
