@@ -1011,7 +1011,7 @@ the arglist, and fn is a symbol, but ~p0 does not satisfy these constraints."
 	       (cadr form)
 	       (cddr form)
 	       (symbol-listp (cddr form)))
-	  (translate-declaration-to-guard-var-lst (cadr form) (cddr form)))
+	  (translate-declaration-to-guard-var-lst (cadr form) (cddr form) nil))
 	 (t ())))))
 
 (defloop get-guards-from-body1 (body)
