@@ -1,7 +1,7 @@
 #|
 
-   Fully Ordered Finite Sets, Version 0.9
-   Copyright (C) 2003, 2004 by Jared Davis <jared@cs.utexas.edu>
+   Fully Ordered Finite Sets, Version 0.91
+   Copyright (C) 2003-2006 by Jared Davis <jared@cs.utexas.edu>
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -119,6 +119,10 @@
                 (not (in a (tail X))))
            (equal (head X) a)))
 
+(defthm in-head
+  (equal (in (head X) X)
+         (not (empty X))))
+
 
 ;;; We now begin to move away from set order.
 
