@@ -351,8 +351,8 @@
 
 (theory-invariant
  (implies
-  (member-equal '(:rewrite cancel-equal-lognot . 2) theory)
-  (member-equal '(:executable-counterpart lognot) theory)))
+  (active-runep '(:rewrite cancel-equal-lognot . 2))
+  (active-runep '(:executable-counterpart lognot))))
 
 ;;;  LOGNOTU
 
