@@ -1,5 +1,9 @@
 ;  Copyright (C) 2000 Panagiotis Manolios and J Strother Moore
  
+; Modified May 2004 by Matt Kaufmann, in order to allow stobjs.  Thanks to John
+; Matthews for providing a motivating example.  NOTE:  Do not use a :stobjs
+; declaration in your defpun!
+
 ;  This program is free software; you can redistribute it and/or modify
 ;  it under the terms of the GNU General Public License as published by
 ;  the Free Software Foundation; either version 2 of the License, or
@@ -95,6 +99,9 @@
 
 (in-package "ACL2")
 
+(defmacro defun-nonexec (name args &rest rst)
+  `(defun ,name ,args (declare (xargs :non-executable t)) ,@rst))
+
 (defstub test (x) t)
 (defstub base (x) t)
 (defstub st (x) t)
@@ -192,7 +199,7 @@
     
       (local (in-theory (disable ,test ,base ,st)))
 
-      (local (defun ,stn (x n)
+      (local (defun-nonexec ,stn (x n)
                (if (zp n)
                    x
                  (,stn (,st x) (1- n)))))
@@ -200,14 +207,14 @@
       (local (defchoose ,fch (n) (x)
                (,test (,stn x n))))
 
-      (local (defun ,fn (x n)
+      (local (defun-nonexec ,fn (x n)
                (declare (xargs :measure (nfix n)))
                (if (or (zp n)
                        (,test x))
                    (,base x) 
                  (,fn (,st x) (1- n)))))
 
-      (local (defun ,f (x)
+      (local (defun-nonexec ,f (x)
                (if (,test (,stn x (,fch x)))
                    (,fn x (,fch x))
                  nil)))
@@ -292,11 +299,11 @@
        ((,f ,vars t))
        (set-ignore-ok t)
        (set-irrelevant-formals-ok t)
-       (local (defun ,test1 (x) (let ,bindings ,test-body)))
-       (local (defun ,base1 (x) (let ,bindings ,base-body)))
-       (local (defun ,step1 (x) (let ,bindings ,step-body)))
+       (local (defun-nonexec ,test1 (x) (let ,bindings ,test-body)))
+       (local (defun-nonexec ,base1 (x) (let ,bindings ,base-body)))
+       (local (defun-nonexec ,step1 (x) (let ,bindings ,step-body)))
        (local ,(arity-1-tail-recursive-encap f1 test1 base1 step1))
-       (local (defun ,f ,vars (,f1 (list ,@vars))))
+       (local (defun-nonexec ,f ,vars (,f1 (list ,@vars))))
        (defthm ,(packn-pos (list f "-DEF") f)
          (equal (,f ,@vars) ,body)
          :hints (("Goal" :use (:instance ,(packn-pos (list f1 "-DEF") f)
@@ -443,7 +450,7 @@
                    ('declare ('xargs ':witness &))))
        `(encapsulate
          ((,g ,vars t))
-         (local (defun ,g ,vars (,(caddr (cadr dcl))
+         (local (defun-nonexec ,g ,vars (,(caddr (cadr dcl))
                                  ,@vars)))
          (defthm ,(packn-pos (list g "-DEF") g)
            (equal (,g ,@vars)
@@ -474,7 +481,7 @@
                   'undef))
               (encapsulate
                ((,g ,vars t))
-               (local (defun ,g ,vars (,gwit ,@vars)))
+               (local (defun-nonexec ,g ,vars (,gwit ,@vars)))
                (defthm ,(packn-pos (list g "-DEF") g)
                  (implies ,dom-expr
                           (equal (,g ,@vars)
@@ -488,7 +495,7 @@
               (verify-guards ,gwit))))
          (t `(encapsulate
               ((,g ,vars t))
-              (local (defun ,g ,vars
+              (local (defun-nonexec ,g ,vars
                        (declare (xargs :measure
                                        (if ,dom-expr
                                            ,measure-expr
