@@ -12,12 +12,12 @@
         (t (cons (car l)
                  (firstn (1- n) (cdr l))))))
 
-(defun lastn (n a)
+(defun dropn (n a)
   "Behaves similarly to nthcdr"
   (declare (xargs :guard (and (natp n))))
   (if (or (atom a) (zp n))
       a
-    (lastn (1- n) (cdr a))))
+    (dropn (1- n) (cdr a))))
 
 (defun wcmp (a b)
   "A comparitor function for numbers"
@@ -170,7 +170,7 @@
   "Does the same as count1 if n < (count1 a b)"
   (declare (xargs :guard (and (o-p x) (o-p y) (natp n))
 		  :verify-guards nil))
-  (+ n (count1 (lastn n x) y)))
+  (+ n (count1 (dropn n x) y)))
 
 (defun padd (x y n)
   "Pseudo-add function used in pmult"
@@ -202,7 +202,7 @@
 		     fco-y
 		     (pmult x (o-rst y) m))))))
 
-(defexec ob* (x y)
+(defun ob* (x y)
   "Efficient ordinal multiplication"
   (declare (xargs :guard (and (o-p x) (o-p y))
 		  :verify-guards nil))
@@ -296,7 +296,7 @@
 		  1)
       (o^3 a (natpart b))))
 
-(defexec ob^ (x y)
+(defun ob^ (x y)
   "Efficient ordinal exponentiation"
   (declare (xargs :guard (and (o-p x) (o-p y))
 		  :verify-guards nil))
