@@ -7,6 +7,8 @@
 
 (in-package "ACL2")
 
+(set-enforce-redundancy t)
+
 (local (include-book "../arithmetic/fp"))
 (local (include-book "../arithmetic/fp2"))
 (local (include-book "../arithmetic/fl"))
@@ -733,21 +735,7 @@
                 (case-split (not (equal r 0)))
                 )
            (equal (expt r (+ i j))
-                  (* (expt r i) (expt r j))))
-  :hints (("Goal" :in-theory (enable expt)))
-  )
-
-#| expt-split replaced this:
-(defthm my-exponents-add
-  (implies (and (not (equal 0 r))
-                (acl2-numberp r)
-                (integerp i)
-                (integerp j))
-           (equal (expt r (+ i j))
-                  (* (expt r i)
-                     (expt r j))))
-  :rule-classes nil)
-|#
+                  (* (expt r i) (expt r j)))))
 
 (defthm cancel-<-*
   (implies (and (rationalp r)
