@@ -58,14 +58,18 @@ stuff may happen (but we can easily handle these cases).
 
 ;In proofs about RTL terms, i and j should always be natural number constants
 
-(defun bits (x i j)
-  (declare (xargs :guard (rationalp x)))
-  (if (or (not (integerp i))
-          (not (integerp j)))
-      0
-  (fl (/ (mod x (expt 2 (1+ i))) (expt 2 j)))))
-
-(in-theory (disable bits))
+(defund bits (x i j)
+  (declare (xargs :guard (and (natp x)
+                              (natp i)
+                              (natp j))
+                  :verify-guards nil))
+  (mbe :logic (if (or (not (integerp i))
+                      (not (integerp j)))
+                  0
+                (fl (/ (mod x (expt 2 (1+ i))) (expt 2 j))))
+       :exec  (if (< i j)
+                  0
+                (logand (ash x (- j)) (1- (ash 1 (1+ (- i j))))))))
 
 (defthm bits-nonnegative-integerp-type
   (and (<= 0 (bits x i j))
@@ -109,14 +113,14 @@ stuff may happen (but we can easily handle these cases).
 
 ;has the right pattern to rewrite stuff like this: (<= (EXPT 2 J) (BITS Y (+ -1 J) 0)) to nil
 (defthm bits-upper-bound-special
-  (< (BITS x (+ -1 i) 0) (EXPT 2 i))
-  :hints (("Goal" :use (:instance bits-upper-bound (i (+ -1 i)) (j 0))))
+  (< (BITS x (1- i) 0) (EXPT 2 i))
+  :hints (("Goal" :use (:instance bits-upper-bound (i (1- i)) (j 0))))
   )
 
 ;tigher bound
 (defthm bits-upper-bound-tighter
   (implies (case-split (<= j i))
-           (<= (bits x i j) (+ -1 (expt 2 (+ i 1 (- j))))))
+           (<= (bits x i j) (1- (expt 2 (+ i 1 (- j))))))
   :rule-classes (:rewrite (:linear :trigger-terms ((bits x i j))))
   :hints (("Goal" :cases ((rationalp x) (not (acl2-numberp x)))
            :in-theory (enable bits 
@@ -288,7 +292,7 @@ stuff may happen (but we can easily handle these cases).
            (equal (bits x i 1)
                   (if (equal (bitn x 0) 0)
                       (/ (bits x i 0) 2) ;use the fact that we know bit 0 is 0?
-                    (/ (+ -1 (bits x i 0)) 2) ;use the fact that we know bit 0 is 1?
+                    (/ (1- (bits x i 0)) 2) ;use the fact that we know bit 0 is 1?
                     )))
   :otf-flg t
   :hints (("Goal" :in-theory (enable bits mod bitn))))
@@ -315,7 +319,8 @@ stuff may happen (but we can easily handle these cases).
 ;(local (in-theory (disable expt-inverse)))
 
 (defthm bits-bvecp-simple
-  (bvecp (bits x i j) (+ 1 i (* -1 j)))
+  (implies (equal k (+ 1 i (* -1 j)))
+           (bvecp (bits x i j) k))
   :hints (("Goal" :in-theory (enable 
                               bvecp))))
 
@@ -326,7 +331,8 @@ stuff may happen (but we can easily handle these cases).
                 (case-split (integerp k))
                 )
            (bvecp (bits x i j) k))
-  :hints (("Goal" :use bits-bvecp-simple
+  :hints (("Goal" :use ((:instance bits-bvecp-simple
+                                   (k (+ 1 i (* -1 j)))))
            :in-theory (disable bits-bvecp-simple))))
 
 (in-theory (disable bits-bvecp-simple))
@@ -651,7 +657,7 @@ equality in which each side should have a power of 2 divided out of it
 ;Recall that <= gets rewritten to < during proofs
 (defthm bits-drop-silly-upper-bound
   (implies (and (syntaxp (quotep k))
-                (>= k (+ -1 (expt 2 (+ 1 i (- j)))))
+                (>= k (1- (expt 2 (+ 1 i (- j)))))
                 (case-split (<= j i))
 ;                (case-split (rationalp x))
                 (case-split (integerp i))
@@ -665,7 +671,7 @@ equality in which each side should have a power of 2 divided out of it
 ;Recall that <= gets rewritten to < during proofs
 (defthm bits-drop-silly-lower-bound
   (implies (and (syntaxp (quotep k))
-                (> k (+ -1 (expt 2 (+ 1 i (- j)))))
+                (> k (1- (expt 2 (+ 1 i (- j)))))
                 (case-split (<= j i))
 ;                (case-split (rationalp x))
                 (case-split (integerp i))
@@ -879,7 +885,7 @@ equality in which each side should have a power of 2 divided out of it
 
 (defthm bits-with-bad-index-2
   (IMPLIES (NOT (INTEGERP i))
-           (EQUAL (BITS x (+ -1 i) 0)
+           (EQUAL (BITS x (1- i) 0)
                   0))
   :hints (("Goal" :in-theory (enable bits))))
 
@@ -906,7 +912,7 @@ equality in which each side should have a power of 2 divided out of it
 		(bvecp x n)
 		(bvecp y n)
 		)
-	   (equal (bits (+ x (* -1 y)) (+ -1 n) 0)
+	   (equal (bits (+ x (* -1 y)) (1- n) 0)
 		  (+ x (* -1 y))
 		  ))
   :hints (("Goal" :cases ((integerp n))
@@ -923,7 +929,7 @@ equality in which each side should have a power of 2 divided out of it
 
 (defthm bits-tail-special
    (implies (bvecp x i)
-            (equal (bits x (+ -1 i) 0)
+            (equal (bits x (1- i) 0)
                    x))
    :hints (("Goal" :cases ((acl2-numberp i)))))
 
@@ -959,7 +965,7 @@ comments from bits.lisp (these may duplicate comments in this book)
            (equal (bits x i 1)
                   (if (equal (bitn x 0) 0)
                       (/ (bits x i 0) 2) ;use the fact that we know bit 0 is 0?
-                    (/ (+ -1 (bits x i 0)) 2) ;use the fact that we know bit 0 is 1?
+                    (/ (1- (bits x i 0)) 2) ;use the fact that we know bit 0 is 1?
                     ))))
 
 
@@ -1224,7 +1230,7 @@ equality in which each side should have a power of 2 divided out of it
 
 
 (defthm bits-bvecp-simple-2
-  (bvecp (bits x (+ -1 i) 0) i)
+  (bvecp (bits x (1- i) 0) i)
   :hints (("Goal" :cases ((acl2-numberp i)))))
 
 
@@ -1239,7 +1245,19 @@ equality in which each side should have a power of 2 divided out of it
                    (bits x n m)))
    :hints (("Goal" :cases ((integerp n))))) ;why cases hint needed?
 
- (defthm bits-plus-bits
+(defthmd bits-plus-mult-2-rewrite
+   (implies (and (syntaxp (quotep c))
+                 (equal (mod c (expt 2 (1+ n))) 0))
+            (equal (bits (+ c x) n m)
+                   (bits x n m)))
+    :hints (("Goal" :use ((:instance bits-plus-mult-2
+                                     (x x)
+                                     (y (/ c (expt 2 (1+ n))))
+                                     (k (1+ n))
+                                     (n n)))
+             :in-theory (enable mod))))
+
+(defthm bits-plus-bits
    (implies (and (integerp m)
                  (integerp p)
                  (integerp n)
@@ -1375,12 +1393,12 @@ equality in which each side should have a power of 2 divided out of it
 ;rename
 ;Follows from BVECP-SUM-OF-BVECPS.
 (defthm bits-sum-1
-  (equal (bits (+ (bits x (+ -1 i) 0)
-                  (bits y (+ -1 i) 0))
+  (equal (bits (+ (bits x (1- i) 0)
+                  (bits y (1- i) 0))
                i ;actually, this could be anything >= i ??
                0)
-         (+ (bits x (+ -1 i) 0)
-            (bits y (+ -1 i) 0)))
+         (+ (bits x (1- i) 0)
+            (bits y (1- i) 0)))
   :hints (("Goal" :in-theory (enable bits-tail))))
 
 
@@ -1388,7 +1406,7 @@ equality in which each side should have a power of 2 divided out of it
 ;gen?
 (defthmd bits-of-non-integer-special
   (implies (case-split (not (integerp i)))
-           (equal (BITS X (+ -1 i) 0)
+           (equal (BITS X (1- i) 0)
                   0)))
 
 (local
@@ -1504,7 +1522,7 @@ equality in which each side should have a power of 2 divided out of it
                 (acl2-numberp j)
                 )
            (equal (bits (* 2 x) i j)
-                  (bits x (+ -1 i) (+ -1 j))))
+                  (bits x (1- i) (1- j))))
   :hints (("Goal" :use ((:instance bits-shift (n 1))))))
 
 (defthmd bits+2**k-2
@@ -1647,7 +1665,7 @@ equality in which each side should have a power of 2 divided out of it
                 (integerp j)
                 (integerp i))
            (equal (mod (bits x i 0) (expt 2 j))
-                  (bits x (min i (+ -1 j)) 0)))
+                  (bits x (min i (1- j)) 0)))
   :hints (("Goal" :in-theory (enable bits)))
   )
 
@@ -1681,11 +1699,11 @@ equality in which each side should have a power of 2 divided out of it
 (defthm bits-minus-better-helper-2
   (implies (and (integerp x)
                 (integerp i))
-           (equal (equal 0 (bits x (+ -1 j) 0))
+           (equal (equal 0 (bits x (1- j) 0))
                   (integerp (* x (/ (expt 2 j))))
                   ))
   :hints (("Goal" :in-theory (enable expt-split mod-equal-0)
-           :expand   (bits x (+ -1 j) 0)))
+           :expand   (bits x (1- j) 0)))
   )
 
 ;note that although the RHS looks slightly gross, 
@@ -1699,7 +1717,7 @@ equality in which each side should have a power of 2 divided out of it
            (equal (bits (* -1 x) i j)
                   (if (equal 0 (bits x i 0))
                       0
-                    (if (equal 0 (bits x (+ -1 j) 0))
+                    (if (equal 0 (bits x (1- j) 0))
                         (+ (* 2 (expt 2 i) (/ (expt 2 j))) (- (bits x i j)))
                       (+ -1 (* 2 (expt 2 i) (/ (expt 2 j))) (- (bits x i j)))))))
 	:hints (("Goal" :use bits-minus
@@ -1720,7 +1738,7 @@ equality in which each side should have a power of 2 divided out of it
                 (case-split (integerp n))
                 )
            (equal (bits x n j)
-                  (bits x (+ -1 i) j)))
+                  (bits x (1- i) j)))
   :rule-classes ((:rewrite :match-free :all))
   :hints (("goal" :use (:instance expt-weak-monotone (n i) (m (+ 1 n)))
-           :in-theory (e/d (bits bvecp) (expt-compare)))))
\ No newline at end of file
+           :in-theory (e/d (bits bvecp) (expt-compare)))))
