@@ -16,15 +16,24 @@
   (floor x 1))
 
 (defund bits (x i j)
-  (declare (xargs :guard (rationalp x)))
-  (if (or (not (integerp i))
-          (not (integerp j)))
-      0
-    (fl (/ (mod x (expt 2 (1+ i))) (expt 2 j)))))
+  (declare (xargs :guard (and (natp x)
+                              (natp i)
+                              (natp j))
+                  :verify-guards nil))
+  (mbe :logic (if (or (not (integerp i))
+                      (not (integerp j)))
+                  0
+                (fl (/ (mod x (expt 2 (1+ i))) (expt 2 j))))
+       :exec  (if (< i j)
+                  0
+                (logand (ash x (- j)) (1- (ash 1 (1+ (- i j))))))))
 
 (defund bitn (x n)
-  (declare (xargs :guard (rationalp x)))
-  (bits x n n))
+  (declare (xargs :guard (and (natp x)
+                              (natp n))
+                  :verify-guards nil))
+  (mbe :logic (bits x n n)
+       :exec  (if (evenp (ash x (- n))) 0 1)))
 
 ;(defund ocat (x y n)
 ;  (declare (xargs :guard t))
@@ -34,7 +43,7 @@
   (declare (xargs :guard (and (integerp n) (<= 0 n))))
   (if (zp n)
       0 ;degenerate case
-    (+ -1 (expt 2 n))))
+    (1- (expt 2 n))))
 
 (include-book "cat-def")
 (local (include-book "../arithmetic/top")) ;try
@@ -86,13 +95,13 @@ Concatenate the M-bit value X onto the N-bit value Y.  X will occupy the high bi
                 (case-split (integerp m))
                 (case-split (<= 0 m))
                 )		  
-           (equal (cat 0 m y n) (bits y (+ -1 n) 0)))
+           (equal (cat 0 m y n) (bits y (1- n) 0)))
   :hints (("Goal" :in-theory (enable cat bits-tail))))
 
 ;We can rely on bits-tail to complete the simplification down to x if desired.
 (defthm cat-with-n-0
   (equal (binary-cat x m y 0)
-         (bits x (+ -1 m) 0))
+         (bits x (1- m) 0))
   :hints (("goal" :in-theory (enable cat))))
 
 (local (in-theory (enable bits-tail)))
@@ -105,7 +114,7 @@ Concatenate the M-bit value X onto the N-bit value Y.  X will occupy the high bi
 ;We can rely on bits-tail to complete the simplification down to y if desired.
 (defthm cat-with-m-0
   (equal (binary-cat x 0 y n)
-         (bits y (+ -1 n) 0))
+         (bits y (1- n) 0))
   :hints (("goal" :in-theory (enable cat))))
 
 (defthm cat-with-m-0-alt
@@ -138,8 +147,8 @@ Concatenate the M-bit value X onto the N-bit value Y.  X will occupy the high bi
 		   (< n 0)
 		   )
 	       0
-	     (ocat (bits x (+ -1 m) 0)
-		   (bits y (+ -1 n) 0)
+	     (ocat (bits x (1- m) 0)
+		   (bits y (1- n) 0)
 		   n)))
   :hints (("Goal" :in-theory (enable ocat cat))))
 |#
@@ -158,7 +167,7 @@ Concatenate the M-bit value X onto the N-bit value Y.  X will occupy the high bi
                               (enable bvecp cat expt-split)
                               '(expt-compare EXPT-COMPARE-EQUAL ))
 		  :use ((:instance expt-split (i m) (j n) (r 2))
-			(:instance hack-10 (x (BITS X (+ -1 M) 0)) (y (expt 2 m)))
+			(:instance hack-10 (x (BITS X (1- M) 0)) (y (expt 2 m)))
 			(:instance expt-weak-monotone (m p))
 			(:instance expt-weak-monotone (m k) (n (+ m n)))))))
 
@@ -325,7 +334,7 @@ these aren't right any more?
                 (case-split (integerp n))
                 )
            (<= (cat x m y n)
-               (+ -1 (expt 2 (+ n m)))))
+               (1- (expt 2 (+ n m)))))
   :hints (("goal" :use cat-upper-bound
            :in-theory (set-difference-theories
                        (enable)
@@ -334,7 +343,7 @@ these aren't right any more?
 
 (defthm cat-compare-to-constant-2
   (implies (and (syntaxp (quotep k))
-                (<= (+ -1 (expt 2 (+ m n))) k)
+                (<= (1- (expt 2 (+ m n))) k)
                 (case-split (<= 0 m))
                 (case-split (<= 0 n))
                 (case-split (integerp m))
@@ -393,7 +402,7 @@ these aren't right any more?
                         (integerp n)
                         )
                    (implies (bvecp (cat x m y n) k)
-                            (bvecp (bits x (+ -1 m) 0) (+ k (* -1 n))) ;the BITS call around x is weird but necessary
+                            (bvecp (bits x (1- m) 0) (+ k (* -1 n))) ;the BITS call around x is weird but necessary
                             ))
           :rule-classes nil
           :otf-flg t
@@ -407,7 +416,7 @@ these aren't right any more?
                         (integerp n)
                         )
                    (implies (bvecp (cat x m y n) k)
-                            (equal 0 (bits x (+ -1 m) (+ k (* -1 n))))
+                            (equal 0 (bits x (1- m) (+ k (* -1 n))))
                             ))
           :rule-classes nil
           :otf-flg t
@@ -416,7 +425,7 @@ these aren't right any more?
 ;move!
 ;this can help, especially when we aren't multiplying through by inverted factors
  (defthm bits-upper-bound-new
-   (< (* (/ (expt 2 i)) (bits x (+ -1 i) 0)) 1)
+   (< (* (/ (expt 2 i)) (bits x (1- i) 0)) 1)
    :hints (("Goal" :in-theory (disable expt-inverse)))
    :rule-classes (:rewrite :linear)
    )
@@ -427,7 +436,7 @@ these aren't right any more?
                         (< k (+ m n))
                         (integerp k)
                         )
-                   (implies (bvecp (bits x (+ -1 m) 0) (- k n))
+                   (implies (bvecp (bits x (1- m) 0) (- k n))
                             (bvecp (cat x m y n) k))) ;the BITS call around x is weird but necessary?
           :rule-classes nil
           :hints (("Goal" :in-theory (e/d (cat bvecp expt-split
@@ -446,7 +455,7 @@ these aren't right any more?
                         (< k (+ m n))
                         (integerp k)
                         )
-                   (implies (equal 0 (bits x (+ -1 m) (+ k (* -1 n))))
+                   (implies (equal 0 (bits x (1- m) (+ k (* -1 n))))
                             (bvecp (cat x m y n) k))) ;the BITS call around x is weird but necessary?
           :rule-classes nil
           :hints (("Goal":use  cat-bvecp-rewrite-case-1-b))))
@@ -460,7 +469,7 @@ these aren't right any more?
                         (case-split (integerp n))
                         )
                    (equal (bvecp (cat x m y n) k)
-                          (equal 0 (bits x (+ -1 m) (+ k (* -1 n)))))) ;the BITS call around x is weird but necessary?
+                          (equal 0 (bits x (1- m) (+ k (* -1 n)))))) ;the BITS call around x is weird but necessary?
           :hints (("Goal" :use ( cat-bvecp-rewrite-case-1-b-better  cat-bvecp-rewrite-case-1-a-better)))
           ))
 
@@ -477,8 +486,8 @@ these aren't right any more?
                         (integerp k)
                         )
                    (implies (bvecp (cat x m y n) k)
-                            (and (equal (bits x (+ -1 m) 0) 0)
-                                 (bvecp (bits y (+ -1 n) 0) k))))
+                            (and (equal (bits x (1- m) 0) 0)
+                                 (bvecp (bits y (1- n) 0) k))))
           :rule-classes nil
           :otf-flg t
           :hints (("Goal"
@@ -498,8 +507,8 @@ these aren't right any more?
                         (integerp k)
                         )
                    (implies (bvecp (cat x m y n) k)
-                            (and (equal 0 (bits x (+ -1 m) 0))
-                                 (equal 0 (bits y (+ -1 n) k)))))
+                            (and (equal 0 (bits x (1- m) 0))
+                                 (equal 0 (bits y (1- n) k)))))
           :rule-classes nil
           :otf-flg t
           :hints (("Goal"
@@ -511,8 +520,8 @@ these aren't right any more?
                         (integerp n)
                         (integerp k)
                         )
-                   (implies (and (equal (bits x (+ -1 m) 0) 0)
-                                 (bvecp (bits y (+ -1 n) 0) k))
+                   (implies (and (equal (bits x (1- m) 0) 0)
+                                 (bvecp (bits y (1- n) 0) k))
                             (bvecp (cat x m y n) k)))
           :rule-classes nil
           :otf-flg t
@@ -536,8 +545,8 @@ these aren't right any more?
                         (integerp n)
                         (integerp k)
                         )
-                   (implies (and (equal (bits x (+ -1 m) 0) 0)
-                                 (equal 0 (bits y (+ -1 n) k)))
+                   (implies (and (equal (bits x (1- m) 0) 0)
+                                 (equal 0 (bits y (1- n) k)))
                             (bvecp (cat x m y n) k)))
           :rule-classes nil
           :otf-flg t
@@ -553,8 +562,8 @@ these aren't right any more?
                         (integerp k)
                         )
                    (equal (bvecp (cat x m y n) k)
-                          (and (equal 0 (bits x (+ -1 m) 0))
-                               (equal 0 (bits y (+ -1 n) k)))))
+                          (and (equal 0 (bits x (1- m) 0))
+                               (equal 0 (bits y (1- n) k)))))
           :otf-flg t
           :hints (("Goal"
                    :use (cat-bvecp-rewrite-case-2-a-better
@@ -574,9 +583,9 @@ these aren't right any more?
                    (if (<= (+ m n) k)
                        t
                      (if (<= n k)
-                         (equal 0 (bits x (+ -1 m) (+ k (* -1 n))))                      
-                       (and (equal 0 (bits x (+ -1 m) 0))
-                            (equal 0 (bits y (+ -1 n) k))))))))
+                         (equal 0 (bits x (1- m) (+ k (* -1 n))))                      
+                       (and (equal 0 (bits x (1- m) 0))
+                            (equal 0 (bits y (1- n) k))))))))
 
  ) ;end the encapsulate
 
@@ -593,9 +602,9 @@ these aren't right any more?
                   (if (<= (+ m n) k)
                       t
                     (if (<= n k)
-                        (equal 0 (bits x (+ -1 m) (+ k (* -1 n))))                      
-                      (and (equal 0 (bits x (+ -1 m) 0))
-                           (equal 0 (bits y (+ -1 n) k)))))))
+                        (equal 0 (bits x (1- m) (+ k (* -1 n))))                      
+                      (and (equal 0 (bits x (1- m) 0))
+                           (equal 0 (bits y (1- n) k)))))))
   :hints (("Goal" :by cat-bvecp-rewrite)))
 
 
@@ -641,7 +650,7 @@ Art's example:
                 (case-split (<= 0 n)))
            (equal (equal y (binary-cat x m y n))
                   (equal (bits y (+ -1 m n) n)
-                         (bits x (+ -1 m) 0))))
+                         (bits x (1- m) 0))))
   :hints (("Goal" :in-theory (enable cat a15)
            :use (:instance bits-plus-bits (x y) (n  (+ -1 m n)) (m 0) (p n))))
   )
@@ -654,7 +663,7 @@ Art's example:
            (equal (equal y (binary-cat x m y n))
                   (if (bvecp y (+ m n))
                       (equal (bits y (+ -1 m n) n)
-                             (bits x (+ -1 m) 0))
+                             (bits x (1- m) 0))
                     nil)))
   :hints (("Goal" :in-theory (disable  cat-equal-y)
            :use  cat-equal-y))
@@ -667,8 +676,8 @@ Art's example:
                 (case-split (<= 0 m))
                 (case-split (integerp n))
                 (case-split (<= 0 n)))
-           (equal (equal (bits y (+ -1 n) 0) (binary-cat x m y n))
-                  (equal (bits x (+ -1 m) 0) 0)))
+           (equal (equal (bits y (1- n) 0) (binary-cat x m y n))
+                  (equal (bits x (1- m) 0) 0)))
   :hints (("goal" :in-theory (enable binary-cat)))
    )
 
@@ -681,15 +690,15 @@ Art's example:
                 (case-split (integerp n))
                 (case-split (<= 0 n)))
            (equal (equal y (binary-cat x m y n))
-                  (equal 0 (bits x (+ -1 m) 0))))
+                  (equal 0 (bits x (1- m) 0))))
   :hints (("Goal" :use cat-equal-bits-of-y)) ;drop this hint
   )
 
 ;enable?
 ;make into 2 separate lemmas (can drop the bits from x or from y)
 (defthmd cat-ignores-bits
-  (equal (cat (bits x (+ -1 m) 0)
-              m (bits y (+ -1 n) 0)
+  (equal (cat (bits x (1- m) 0)
+              m (bits y (1- n) 0)
               n)
          (cat x m y n))
   :hints (("goal" :in-theory (enable cat))))
@@ -744,7 +753,7 @@ Art's example:
            (equal (bits (cat x m y n) i j)
                   (bits x (if (< i (+ m n))
                               (- i n)
-                            (+ -1 m))
+                            (1- m))
                         (- j n))))
   :hints (("Goal" :in-theory (enable cat)))
   )
@@ -762,7 +771,7 @@ Art's example:
            (equal (bits (cat x m y n) i j)
                   (cat (bits x (if (< i (+ m n))
                                     (- i n)
-                                  (+ -1 m))
+                                  (1- m))
                               0)
                         (+ 1 (- i n))
                         (bits y (1- n) j)
@@ -784,11 +793,11 @@ Art's example:
                     (if (>= j n)
                         (bits x (if (< i (+ m n))
                                     (- i n)
-                                  (+ -1 m)) 
+                                  (1- m)) 
                               (- j n))
                       (cat (bits x (if (< i (+ m n))
                                         (- i n)
-                                      (+ -1 m)) 0)
+                                      (1- m)) 0)
                             (+ 1 (- i n))
                             (bits y (1- n) j)
                             (- n j))))))
@@ -813,11 +822,11 @@ Art's example:
                     (if (>= j n)
                         (bits x (if (< i (+ m n))
                                     (- i n)
-                                  (+ -1 m)) 
+                                  (1- m)) 
                               (- j n))
                       (cat (bits x (if (< i (+ m n))
                                        (- i n)
-                                     (+ -1 m)) 0)
+                                     (1- m)) 0)
                            (+ 1 (- i n))
                            (bits y (1- n) j)
                            (- n j)))))))
@@ -944,8 +953,8 @@ Art's example:
                 (case-split (<= 0 n))
                 )
            (equal (equal k (cat x m y n))
-                  (and (equal (bits y (+ -1 n) 0) (bits k (+ -1 n) 0))
-                       (equal (bits x (+ -1 m) 0) (bits k (+ -1 m n) n)))))
+                  (and (equal (bits y (1- n) 0) (bits k (1- n) 0))
+                       (equal (bits x (1- m) 0) (bits k (+ -1 m n) n)))))
   :hints (("Goal" :in-theory (enable cat-ignores-bits)
            :use ((:instance cat-bits-bits (x  (cat x m y n)) (i (+ -1 m n)) (j n) (k (+ -1 n)) (l 0))))))
 
@@ -969,7 +978,7 @@ Art's example:
                 (case-split (<= 0 n))
                 )
            (equal (equal k (cat x m y n))
-                  (and (equal y (bits k (+ -1 n) 0))
+                  (and (equal y (bits k (1- n) 0))
                        (equal x (bits k (+ -1 m n) n)))))
   :otf-flg t
   :hints (("Goal" :in-theory (enable cat-split-equality))))
@@ -1015,7 +1024,7 @@ Art's example:
                        (+ 2 i (- k))
                        (cat (bits x k l)
                             (+ 1 k (- l))
-                            (bits x (+ -1 l) j)
+                            (bits x (1- l) j)
                             (+ l (- j)))
                        (+ 1 (- j) k))
                   (bits x i j)))
@@ -1088,7 +1097,7 @@ bits-dont-match can prove things like this:
                 (case-split (integerp n))
                 )
            (equal (equal x (cat (bits x (+ -1 m n) n) m y n))
-                  (equal (bits x (+ -1 n) 0) y))
+                  (equal (bits x (1- n) 0) y))
            )
   :hints (("Goal" :in-theory (disable CAT-BITS-BITS
                                       BITS-SPLIT-AROUND-ZERO)
@@ -1106,7 +1115,7 @@ bits-dont-match can prove things like this:
                 (case-split (integerp n))
                 )
            (equal (equal x (cat (bits x kk n) m y n))
-                  (equal (bits x (+ -1 n) 0) y))
+                  (equal (bits x (1- n) 0) y))
            )
   :hints (("Goal" :in-theory (disable CAT-BITS-BITS
                                       BITS-SPLIT-AROUND-ZERO)
@@ -1175,7 +1184,7 @@ bits-dont-match can prove things like this:
 
 ;keep disabled?
 (defthm cat-when-bits-of-x-are-0
-  (implies (and (EQUAL (BITS X (+ -1 M) 0) 0)
+  (implies (and (EQUAL (BITS X (1- M) 0) 0)
                 (integerp m)
                 (<= 0 m))
            (equal (cat x m y n)
