@@ -323,7 +323,7 @@ See also merge2.lisp.
                   (:instance expo>= (n (max n (max (1+ (expo x)) (1+ (expo y))))) (x y)))))))
 
 (defthm exact-bits-1 
-  (implies (and (equal (expo x) (+ -1 n))
+  (implies (and (equal (expo x) (1- n))
                 (rationalp x)
                 (integerp k)
                 )
@@ -334,13 +334,13 @@ See also merge2.lisp.
 
 ;strange rule?
 (defthm exact-bits-2 ; exact-bits-a-c
-  (implies (and (equal (expo x) (+ -1 n))
+  (implies (and (equal (expo x) (1- n))
                 (rationalp x)
                 (<= 0 x) ;or put abs in conclusion?
                 (integerp k)
                 )
            (equal (integerp (/ x (expt 2 k)))
-                  (equal (bits x (+ -1 n) k)
+                  (equal (bits x (1- n) k)
                          (/ x (expt 2 k)))))
   :rule-classes ()
   :hints (("goal" :in-theory (enable bits)
@@ -360,19 +360,19 @@ See also merge2.lisp.
 (defthm exact-bits-3
   (implies (integerp x)
            (equal (integerp (/ x (expt 2 k)))
-                  (equal (bits x (+ -1 k) 0)
+                  (equal (bits x (1- k) 0)
                          0)))
   :rule-classes ()
   :hints (("goal" :in-theory (enable bits)
            :use ((:instance quot-mod (m x) (n (expt 2 k)))))))
 
 (defthm exact-bits-b-d
-  (implies (and (equal (expo x) (+ -1 n))
+  (implies (and (equal (expo x) (1- n))
                 (integerp x)
                 (integerp k)
                 )
            (equal (exactp x (- n k))
-                  (equal (bits x (+ -1 k) 0)
+                  (equal (bits x (1- k) 0)
                          0)))
   :rule-classes ()
   :hints (("goal" :use (exact-bits-3 exact-bits-1))))
@@ -1337,7 +1337,7 @@ test of having this commented out:
                 )
            (equal (ocat (bits x i (+ 1 k))
                         (ocat (bits x k l)
-                              (bits x (+ -1 l) j)
+                              (bits x (1- l) j)
                               (+ l (- j)))
                         (+ 1 (- j) k))
                   (bits x i j)))
@@ -2187,11 +2187,11 @@ three bit vectors is 1, we need only consider 1 bit of carry.
 
 ;BOZO This can be kind of expensive. Add back-chain-limit?
 (defthm bvecp-tighten
-  (implies (and (equal (bitn x (+ -1 n)) 0)
+  (implies (and (equal (bitn x (1- n)) 0)
                 (natp n))
            (equal (bvecp x n)
-                  (bvecp x (+ -1 n))))
-  :hints (("Goal" :use (:instance bitn-plus-bits (n (+ -1 n)) (m 0))
+                  (bvecp x (1- n))))
+  :hints (("Goal" :use (:instance bitn-plus-bits (n (1- n)) (m 0))
            :in-theory (e/d (bvecp)
                            (BITS-SPLIT-AROUND-ZERO
                             ;BITS-REDUCE-EXACTP
