@@ -4,6 +4,7 @@
 (include-book "../arithmetic/power2p")
 (include-book "../arithmetic/negative-syntaxp")
 (local (include-book "../support/bitn"))
+(local (include-book "../support/guards"))
 
 (set-inhibit-warnings "theory") ; avoid warning in the next event
 (local (in-theory nil))
@@ -26,11 +27,16 @@
   (floor x 1))
 
 (defund bits (x i j)
-  (declare (xargs :guard (rationalp x)))
-  (if (or (not (integerp i))
-          (not (integerp j)))
-      0
-    (fl (/ (mod x (expt 2 (1+ i))) (expt 2 j)))))
+  (declare (xargs :guard (and (natp x)
+                              (natp i)
+                              (natp j))))
+  (mbe :logic (if (or (not (integerp i))
+                      (not (integerp j)))
+                  0
+                (fl (/ (mod x (expt 2 (1+ i))) (expt 2 j))))
+       :exec  (if (< i j)
+                  0
+                (logand (ash x (- j)) (1- (ash 1 (1+ (- i j))))))))
 
 (defun expo-measure (x)
 ;  (declare (xargs :guard (and (real/rationalp x) (not (equal x 0)))))
@@ -53,8 +59,10 @@
 ;;
 
 (defund bitn (x n)
-  (declare (xargs :guard (rationalp x)))
-  (bits x n n))
+  (declare (xargs :guard (and (natp x)
+                              (natp n))))
+  (mbe :logic (bits x n n)
+       :exec  (if (evenp (ash x (- n))) 0 1)))
 
 (defthm bitn-with-n-not-an-integer
   (implies (not (integerp n))
