@@ -1,6 +1,7 @@
 (in-package "ACL2")
 
 (local (include-book "../support/cat"))
+(local (include-book "../support/guards"))
 
 ;; Necessary defuns
 
@@ -20,21 +21,28 @@
   (floor x 1))
 
 (defund bits (x i j)
-  (declare (xargs :guard (rationalp x)))
-  (if (or (not (integerp i))
-          (not (integerp j)))
-      0
-    (fl (/ (mod x (expt 2 (1+ i))) (expt 2 j)))))
+  (declare (xargs :guard (and (natp x)
+                              (natp i)
+                              (natp j))))
+  (mbe :logic (if (or (not (integerp i))
+                      (not (integerp j)))
+                  0
+                (fl (/ (mod x (expt 2 (1+ i))) (expt 2 j))))
+       :exec  (if (< i j)
+                  0
+                (logand (ash x (- j)) (1- (ash 1 (1+ (- i j))))))))
 
 (defund bitn (x n)
-  (declare (xargs :guard (rationalp x)))
-  (bits x n n))
+  (declare (xargs :guard (and (natp x)
+                              (natp n))))
+  (mbe :logic (bits x n n)
+       :exec  (if (evenp (ash x (- n))) 0 1)))
 
 (defund all-ones (n)
   (declare (xargs :guard (and (integerp n) (<= 0 n))))
   (if (zp n)
       0 ;degenerate case
-    (+ -1 (expt 2 n))))
+    (1- (expt 2 n))))
 
 
 
@@ -52,63 +60,47 @@ Concatenate the M-bit value X onto the N-bit value Y.  X will occupy the high bi
 
 |#
 
-
-; return 0 if m or n isn't a nat (change this bevahior?)
 (defund binary-cat (x m y n)
-  (declare (xargs :guard (and (rationalp x)
-                              (acl2-numberp m)
-                              (rationalp y)
-                              (acl2-numberp n))))
-  (if (or (not (integerp m))
-          (< m 0)
-          (not (integerp n))
-          (< n 0)
-          )
-      0
-    (+ (* (expt 2 n) (bits x (+ -1 m) 0))
-       (bits y (+ -1 n) 0))))
-
-;; The macro cat
+  (declare (xargs :guard (and (natp x)
+                              (natp y)
+                              (integerp m) (< 0 m)
+                              (integerp n) (< 0 n))))
+  (if (and (natp m) (natp n))
+      (+ (* (expt 2 n) (bits x (1- m) 0))
+         (bits y (1- n) 0))
+    0))
 
 (defun formal-+ (x y)
+  ;;an auxiliary function that does not appear in translate-rtl output.
   (declare (xargs :guard t))
   (if (and (acl2-numberp x) (acl2-numberp y))
       (+ x y)
     (list '+ x y)))
 
-;X is a list of alternating data values and sizes.
-;CAT-SIZE returns the sum of the sizes.
-;X must contain at least 1 data/size pair, but we do not
-;need to specify this in the guard for cat-size-fn (and,
-;leaving it out of that guard simplifies the guard proof).
-(defun cat-size-fn (x)
-  (declare (xargs :guard (and (true-listp x)
-                              (evenp (length x)))))
-  (if (endp (rest (rest x)))
-      (second x)
-    (formal-+ (second x)
-              (cat-size-fn (rest (rest x))))))
+;;X is a list of alternating data values and sizes.  CAT-SIZE returns the
+;;formal sum of the sizes.  X must contain at least 1 data/size pair, but we do
+;;not need to specify this in the guard, and leaving it out of that guard
+;;simplifies the guard proof.
 
-(defmacro cat-size (&rest x)
-  (declare (xargs :guard (and (true-listp x)
-                              (evenp (length x))
-                              (>= (length x) 2))))
-  (cat-size-fn x))
-
-;data and sizes alternate thus: (cat x xsize y ysize z zsize ...)  
-(defmacro cat (&rest x)
+(defun cat-size (x)
+  ;;an auxiliary function that does not appear in translate-rtl output.
   (declare (xargs :guard (and (true-listp x) (evenp (length x)))))
-  (cond ((endp x) ;special case 1
-         `0)
-        ((endp (rest (rest x))) ;special case 2
-         `(bits ,(first x) ,(formal-+ -1 (second x)) 0))
-        ((endp (rest (rest (rest (rest x))))) ;this is really the base case
+  (if (endp (cddr x))
+      (cadr x)
+    (formal-+ (cadr x)
+	      (cat-size (cddr x)))))
+
+(defmacro cat (&rest x)
+  (declare (xargs :guard (and x (true-listp x) (evenp (length x)))))
+  (cond ((endp (cddr x))
+         `(bits ,(car x) ,(formal-+ -1 (cadr x)) 0))
+        ((endp (cddddr x))
          `(binary-cat ,@x))
         (t
-         `(binary-cat ,(first x) 
-                       ,(second x) 
-                       (cat ,@(rest (rest x))) 
-                       (cat-size ,@(rest (rest x)))))))
+         `(binary-cat ,(car x) 
+                      ,(cadr x) 
+                      (cat ,@(cddr x)) 
+                      ,(cat-size (cddr x))))))
 
 ;Allows things like (in-theory (disable cat)) to refer to binary-cat.
 (add-macro-alias cat binary-cat)
