@@ -177,14 +177,11 @@ perhaps this isn't worth spending time on.
                   (integerp x)))
   :hints (("Goal" :in-theory (enable mod))))
 
-(defthm mod-with-y-0
-  (equal (mod x 0)
-         (fix x))
-  :hints (("Goal" :in-theory (enable mod))))
-
-(defthm mod-with-x-0
-  (equal (mod 0 y)
-         0)
+(defthm mod-0
+    (and (equal (mod 0 y)
+                0)
+         (equal (mod x 0)
+                (fix x)))
   :hints (("Goal" :in-theory (enable mod))))
 
 (defthm mod-complex-rationalp-rewrite
@@ -1122,12 +1119,6 @@ not done
 ;			(:instance mod+-thm (m (- m n)) (a 1))
                  ))))
 
-;BOZO duplicates one of my rules
-(defthm mod-0
-  (implies (acl2-numberp x)
-           (equal (mod x 0) x))
-  :hints (("Goal" :expand (mod x 0))))
-
 ;We need this here so that ACL2 knows how to make natp-mod into a type-prescription rule.
 (defthm natp-compound-recognizer
   (equal (natp x)
