@@ -157,15 +157,11 @@ perhaps this isn't worth spending time on.
            (<= (mod x y) x))
   :rule-classes (:rewrite (:linear :trigger-terms ((mod x y)))))
 
-
-
-(defthm mod-with-y-0
-  (equal (mod x 0)
-         (fix x)))
-
-(defthm mod-with-x-0
-  (equal (mod 0 y)
-         0))
+(defthm mod-0
+    (and (equal (mod 0 y)
+                0)
+         (equal (mod x 0)
+                (fix x))))
 
 (defthm mod-complex-rationalp-rewrite
   (implies (case-split (rationalp y))
@@ -864,12 +860,6 @@ ex:   (INTEGERP (* (/ (EXPT 2 J))
 	     (= m n))
   :rule-classes ())
 
-;BOZO duplicates one of my rules
-;bad name...
-(defthm mod-0
-  (implies (acl2-numberp x)
-           (equal (mod x 0) x)))
-
 (defthm natp-compound-recognizer
   (equal (natp x)
          (and (integerp x) (<= 0 x)))
