@@ -15,6 +15,7 @@
 ;; this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 ;; Place - Suite 330, Boston, MA 02111-1307, USA.
 
+; Modified 6/18/06 by Matt K. to replace run-times by acl2-oracle.
 
 (in-package "ACL2")
 
@@ -26,7 +27,7 @@
     32-bit-integer-stack
     big-clock-entry
     idates
-    run-times
+    acl2-oracle
     file-clock
     readable-files
     written-files
@@ -69,8 +70,8 @@
                (big-clock-entry state))
         (equal (idates (update-open-input-channels updates state))
                (idates state))
-        (equal (run-times (update-open-input-channels updates state))
-               (run-times state))    
+        (equal (acl2-oracle (update-open-input-channels updates state))
+               (acl2-oracle state))    
         (equal (file-clock (update-open-input-channels updates state))
                (file-clock state))
         (equal (readable-files (update-open-input-channels updates state))
@@ -125,8 +126,8 @@
                (big-clock-entry state))
         (equal (idates (update-open-output-channels updates state))
                (idates state))
-        (equal (run-times (update-open-output-channels updates state))
-               (run-times state))    
+        (equal (acl2-oracle (update-open-output-channels updates state))
+               (acl2-oracle state))    
         (equal (file-clock (update-open-output-channels updates state))
                (file-clock state))
         (equal (readable-files (update-open-output-channels updates state))
@@ -175,8 +176,8 @@
                (big-clock-entry state))
         (equal (idates (update-global-table updates state))
                (idates state))
-        (equal (run-times (update-global-table updates state))
-               (run-times state))    
+        (equal (acl2-oracle (update-global-table updates state))
+               (acl2-oracle state))    
         (equal (file-clock (update-global-table updates state))
                (file-clock state))
         (equal (readable-files (update-global-table updates state))
@@ -214,8 +215,8 @@
                (big-clock-entry state))
         (equal (idates (update-t-stack updates state))
                (idates state))
-        (equal (run-times (update-t-stack updates state))
-               (run-times state))    
+        (equal (acl2-oracle (update-t-stack updates state))
+               (acl2-oracle state))    
         (equal (file-clock (update-t-stack updates state))
                (file-clock state))
         (equal (readable-files (update-t-stack updates state))
@@ -264,8 +265,8 @@
                (big-clock-entry state))
         (equal (idates (update-32-bit-integer-stack updates state))
                (idates state))
-        (equal (run-times (update-32-bit-integer-stack updates state))
-               (run-times state))    
+        (equal (acl2-oracle (update-32-bit-integer-stack updates state))
+               (acl2-oracle state))    
         (equal (file-clock (update-32-bit-integer-stack updates state))
                (file-clock state))
         (equal (readable-files (update-32-bit-integer-stack updates state))
@@ -313,8 +314,8 @@
                updates)
         (equal (idates (update-big-clock-entry updates state))
                (idates state))
-        (equal (run-times (update-big-clock-entry updates state))
-               (run-times state))    
+        (equal (acl2-oracle (update-big-clock-entry updates state))
+               (acl2-oracle state))    
         (equal (file-clock (update-big-clock-entry updates state))
                (file-clock state))
         (equal (readable-files (update-big-clock-entry updates state))
@@ -362,8 +363,8 @@
                (big-clock-entry state))
         (equal (idates (update-idates updates state))
                updates)
-        (equal (run-times (update-idates updates state))
-               (run-times state))    
+        (equal (acl2-oracle (update-idates updates state))
+               (acl2-oracle state))    
         (equal (file-clock (update-idates updates state))
                (file-clock state))
         (equal (readable-files (update-idates updates state))
@@ -391,45 +392,47 @@
                            (x (update-idates updates state)))))))
 
 
-(defthm update-run-times-state-lemmas
+(defthm update-acl2-oracle-state-lemmas
   (implies 
    (state-p1 state)
-   (and (true-listp (update-run-times updates state))
-        (equal (len (update-run-times updates state))
+   (and (true-listp (update-acl2-oracle updates state))
+        (equal (len (update-acl2-oracle updates state))
                (len state))
-        (equal (open-input-channels (update-run-times updates state))
+        (equal (open-input-channels (update-acl2-oracle updates state))
                (open-input-channels state))
-        (equal (open-output-channels (update-run-times updates state))
+        (equal (open-output-channels (update-acl2-oracle updates state))
                (open-output-channels state))
-        (equal (global-table (update-run-times updates state))
+        (equal (global-table (update-acl2-oracle updates state))
                (global-table state))
-        (equal (t-stack (update-run-times updates state))
+        (equal (t-stack (update-acl2-oracle updates state))
                (t-stack state))
-        (equal (32-bit-integer-stack (update-run-times updates state))
+        (equal (32-bit-integer-stack (update-acl2-oracle updates state))
                (32-bit-integer-stack state))
-        (equal (big-clock-entry (update-run-times updates state))
+        (equal (big-clock-entry (update-acl2-oracle updates state))
                (big-clock-entry state))
-        (equal (idates (update-run-times updates state))
+        (equal (idates (update-acl2-oracle updates state))
                (idates state))
-        (equal (run-times (update-run-times updates state))
+        (equal (acl2-oracle (update-acl2-oracle updates state))
                updates)   
-        (equal (file-clock (update-run-times updates state))
+        (equal (file-clock (update-acl2-oracle updates state))
                (file-clock state))
-        (equal (readable-files (update-run-times updates state))
+        (equal (readable-files (update-acl2-oracle updates state))
                (readable-files state))
-        (equal (written-files (update-run-times updates state))
+        (equal (written-files (update-acl2-oracle updates state))
                (written-files state))
-        (equal (read-files (update-run-times updates state))
+        (equal (read-files (update-acl2-oracle updates state))
                (read-files state))
-        (equal (writeable-files (update-run-times updates state))
+        (equal (writeable-files (update-acl2-oracle updates state))
                (writeable-files state))
-        (equal (list-all-package-names-lst (update-run-times updates state))
+        (equal (list-all-package-names-lst (update-acl2-oracle updates state))
                (list-all-package-names-lst state))
-        (equal (user-stobj-alist1 (update-run-times updates state))
+        (equal (user-stobj-alist1 (update-acl2-oracle updates state))
                (user-stobj-alist1 state))))
-  :hints(("Goal" :in-theory (enable update-run-times))))
+  :hints(("Goal" :in-theory (enable update-acl2-oracle))))
 
 
+;;; Matt K., 6/18/06: The following comment about run-times is obsolete, as
+;;; run-times has been replaced by acl2-oracle:
 ; as it turns out, update-run-times-preserves-state-p1 is already a theorem,
 ; apparently provided to us by axioms.lisp or something.  but, the current
 ; version of it is not as strong as below, and has (rational-listp updates) as
@@ -437,18 +440,18 @@
 ; i prefer the stronger version below, so I will go ahead and disable the
 ; previous version and install mine.
 
-(in-theory (disable update-run-times-preserves-state-p1))
+(in-theory (disable update-acl2-oracle-preserves-state-p1))
 
-(defthm update-run-times-preserves-state-p1-stronger
+(defthm update-acl2-oracle-preserves-state-p1-stronger
   (implies (state-p1 state)
-           (equal (state-p1 (update-run-times updates state))
-                  (rational-listp updates)))
-  :hints(("Goal" :in-theory (disable update-run-times
+           (equal (state-p1 (update-acl2-oracle updates state))
+                  (true-listp updates)))
+  :hints(("Goal" :in-theory (disable update-acl2-oracle
                                      statep-functions)
           :use ((:instance state-p1 
                            (x state))
                 (:instance state-p1
-                           (x (update-run-times updates state)))))))
+                           (x (update-acl2-oracle updates state)))))))
 
 
 
@@ -472,8 +475,8 @@
                (big-clock-entry state))
         (equal (idates (update-file-clock updates state))
                (idates state))
-        (equal (run-times (update-file-clock updates state))
-               (run-times state))
+        (equal (acl2-oracle (update-file-clock updates state))
+               (acl2-oracle state))
         (equal (file-clock (update-file-clock updates state))
                updates)
         (equal (readable-files (update-file-clock updates state))
@@ -522,8 +525,8 @@
                (big-clock-entry state))
         (equal (idates (update-written-files updates state))
                (idates state))
-        (equal (run-times (update-written-files updates state))
-               (run-times state))
+        (equal (acl2-oracle (update-written-files updates state))
+               (acl2-oracle state))
         (equal (file-clock (update-written-files updates state))
                (file-clock state))
         (equal (readable-files (update-written-files updates state))
@@ -569,8 +572,8 @@
                (big-clock-entry state))
         (equal (idates (update-read-files updates state))
                (idates state))
-        (equal (run-times (update-read-files updates state))
-               (run-times state))
+        (equal (acl2-oracle (update-read-files updates state))
+               (acl2-oracle state))
         (equal (file-clock (update-read-files updates state))
                (file-clock state))
         (equal (readable-files (update-read-files updates state))
@@ -618,8 +621,8 @@
                (big-clock-entry state))
         (equal (idates (update-list-all-package-names-lst updates state))
                (idates state))
-        (equal (run-times (update-list-all-package-names-lst updates state))
-               (run-times state))
+        (equal (acl2-oracle (update-list-all-package-names-lst updates state))
+               (acl2-oracle state))
         (equal (file-clock (update-list-all-package-names-lst updates state))
                (file-clock state))
         (equal (readable-files (update-list-all-package-names-lst updates state))
@@ -666,8 +669,8 @@
                (big-clock-entry state))
         (equal (idates (update-user-stobj-alist1 updates state))
                (idates state))
-        (equal (run-times (update-user-stobj-alist1 updates state))
-               (run-times state))
+        (equal (acl2-oracle (update-user-stobj-alist1 updates state))
+               (acl2-oracle state))
         (equal (file-clock (update-user-stobj-alist1 updates state))
                (file-clock state))
         (equal (readable-files (update-user-stobj-alist1 updates state))
@@ -700,7 +703,7 @@
                     update-32-bit-integer-stack
                     update-big-clock-entry
                     update-idates
-                    update-run-times
+                    update-acl2-oracle
                     update-file-clock
                     update-written-files
                     update-read-files
