@@ -1,4 +1,4 @@
-; ACL2 Version 3.0 -- A Computational Logic for Applicative Common Lisp
+; ACL2 Version 3.1 -- A Computational Logic for Applicative Common Lisp
 ; Copyright (C) 2006  University of Texas at Austin
 
 ; This version of ACL2 is a descendent of ACL2 Version 1.9, Copyright
@@ -26,10 +26,7 @@
 
 ; This file contains some emacs stuff for ACL2 users.
 
-; NOTE!!!
-; (1) Edit as indicated in the first section ("EDIT THIS SECTION") below in order
-;     to avoid having to respond to a prompt when you load this file.
-; (2) Suggestion: look at the final section, "Some other features you may want."
+; Suggestion: look at the final section, "Some other features you may want."
 
 ;;; CONTENTS OF THIS FILE
 
@@ -37,8 +34,6 @@
 ;;; lower-level details may be found later in the file where the functionality
 ;;; is actually provided.
 
-; EDIT THIS SECTION!
-  ; Set pathname of ACL2 sources directory.
 ; General shell stuff
   ; Starts up a shell buffer, *shell*.
   ; "meta-x new-shell" starts new shell buffers *shell-1*, *shell-2*, ....
@@ -134,7 +129,7 @@
 ; necessary, however, if this file is located in the emacs/ subdirectory of the
 ; ACL2 source directory (as is the case when it is distributed).
 ; Example:
-; (defvar *acl2-sources-dir* "/u/acl2/v3-0/acl2-sources/")
+; (defvar *acl2-sources-dir* "/u/acl2/v2-9/acl2-sources/")
 (defvar *acl2-sources-dir*)
 
 ; Attempt to set *acl2-sources-dir*.
@@ -184,7 +179,8 @@
 			 (setq number-of-other-shells
 			       (+ 1 number-of-other-shells))))
 		(or (getenv "SHELL")
-		    "csh"))))
+		    "csh")))
+  (shell-mode))
 
 ; Avoid killing shell buffers by accident:
 (defun kill-buffer-without-process (name)
@@ -483,7 +479,7 @@ then also ignore case if that argument is positive, else do not ignore case."
 	   (setq *acl2-sources-dir*
 		 (expand-file-name
 		  (read-file-name
-		  "*acl2-sources-dir* (e.g. /u/acl2/v3-0/acl2-sources/): "
+		  "*acl2-sources-dir* (e.g. /u/acl2/v2-9/acl2-sources/): "
 		  nil nil t))))))
     (if (or (equal dir "")
 	    (let ((lastch (aref "abc/" (1- (length "abc/")))))
