@@ -1,4 +1,4 @@
-; ACL2 Version 3.0 -- A Computational Logic for Applicative Common Lisp
+; ACL2 Version 3.1 -- A Computational Logic for Applicative Common Lisp
 ; Copyright (C) 2006  University of Texas at Austin
 
 ; This version of ACL2 is a descendent of ACL2 Version 1.9, Copyright
@@ -56,10 +56,25 @@
 ; We may need a bigger stack than the default, as evidenced by the failure of
 ; the event (verify-guards read-utf8-fast ...) in books/unicode/read-utf8.lisp.
 ; We handle this issue here for GCL, and elsewhere for some other lisps.
+; However, we have seen GCL 2.6.6 on Windows break here, so we skip the stack
+; adjustment for Windows.
 
-#+gcl
+#+(and gcl (not mswindows))
 (progn
   (defvar *acl2-gcl-multiply-stacks-evaluated* nil)
   (when (not *acl2-gcl-multiply-stacks-evaluated*)
     (setq si::*multiply-stacks* 2))
   (setq *acl2-gcl-multiply-stacks-evaluated* t))
+
+; Suggestion from Camm Maguire, 6/28/06 (GCL 2.6.7 and beyond), for improved
+; efficiency; seconded by Bob Boyer.
+#+gcl
+(when (and (boundp 'si::*gcl-major-version*)
+           (boundp 'si::*gcl-minor-version*)
+           (boundp 'si::*gcl-extra-version*)
+           (or (> si::*gcl-major-version* 2)
+               (and (>= si::*gcl-major-version* 2)
+                    (or (> si::*gcl-minor-version* 6)
+                        (and (>= si::*gcl-minor-version* 6)
+                             (>= si::*gcl-extra-version* 7))))))
+  (declaim (ftype (function (seqind t) t) si::set-mv)))
