@@ -1,4 +1,4 @@
-; ACL2 Version 3.0 -- A Computational Logic for Applicative Common Lisp
+; ACL2 Version 3.1 -- A Computational Logic for Applicative Common Lisp
 ; Copyright (C) 2006  University of Texas at Austin
 
 ; This version of ACL2 is a descendent of ACL2 Version 1.9, Copyright
@@ -404,6 +404,10 @@
 ;; to stand for the non-zero numbers.
 
 (defun linearize1 (term positivep type-alist ens force-flg wrld ttree state)
+
+; See the comment in linearize.  Linearize1 does all the work of linearize
+; except that the latter maps normalize-poly over the former.
+
     (mv-let (flg temp ttree)
       (eval-ground-subexpressions term ens wrld state ttree)
       (declare (ignore flg))
@@ -422,7 +426,6 @@
 ; new role with respect to atm.
 
       (let ((positivep (if not-flg (not positivep) positivep)))
-
         (cond
          ((inequalityp atm)
           (let ((lhs (fargn atm 1))
@@ -435,48 +438,79 @@
                 (type-set rhs force-flg nil
                           type-alist nil ens wrld ts-ttree
                           nil nil)
-                (if positivep
-                    ;; (< lhs rhs)
+                (if positivep ; (< lhs rhs)
                     (cond
                      ((and (ts-integerp ts-lhs)
                            (ts-integerp ts-rhs))
-                      ;; (implies (and (< lhs rhs)
-                      ;;               (integerp lhs)
-                      ;;               (integerp rhs))
-                      ;;          (<= (1+ lhs) rhs))
+
+; (implies (and (< lhs rhs)
+;               (integerp lhs)
+;               (integerp rhs))
+;          (<= (1+ lhs) rhs))
+
                       (poly-set 'and
                                 nil
-                                (add-linear-terms :lhs lhs
-                                                  :lhs *1*
-                                                  :rhs rhs
-                                                  (base-poly ts-ttree
-                                                             '<=
-                                                             t
-                                                             nil))))
-                     (t
-                      ;; (< lhs rhs)
+                                (add-linear-terms
+                                 :lhs lhs
+                                 :lhs *1*
+                                 :rhs rhs
+                                 (base-poly ts-ttree
+                                            '<=
+                                            t
+                                            nil))))
+                     (t ; still (< lhs rhs), but not both integerp
                       (poly-set 'and
                                 nil
                                 (add-linear-terms
                                  :lhs lhs
                                  :rhs rhs
-                                 (base-poly ttree
-                                            '<
-                                            (and (ts-rationalp ts-lhs)
-                                                 (ts-rationalp ts-rhs))
-                                            nil)))))
-                  ;; (implies (not (< lhs rhs))
-                  ;;          (<= rhs lhs))
+                                 (let ((rationalp-flg
+                                        (and (ts-rationalp ts-lhs)
+                                             (ts-rationalp ts-rhs))))
+                                   (base-poly0 (if rationalp-flg
+                                                   ts-ttree
+                                                 ttree)
+
+; In a slight break from tradition (here and in three other places below that
+; refer to this comment), we use the parents from the original ttree.  When
+; fixing a (probably long-standing) bug in Version_3.0.1 by recording ts-ttree
+; above in the case that rationalp-flg is true, we found that the regression
+; suite broke in three places without this tweak on the parents.  Since
+; rationalp-flg is only used in non-linear arithmetic, this seems like a minor
+; break from our traditional propagation of parent trees.  We considered making
+; a similar change for all calls of base-poly in this function, but that led to
+; a proof failure in
+; books/workshops/2004/schmaltz-borrione/support/routing_defuns.lisp that
+; looked like it would be painful to fix, and we took that as a sign that such
+; loss of backward compatibility could be painful for other users, and
+; potentially even a bad heuristic.
+
+                                               (collect-parents ttree)
+                                               '<
+                                               rationalp-flg
+                                               nil))))))
+
+; The (not positivep) case.  Note:
+; (implies (not (< lhs rhs))
+;          (<= rhs lhs))
+
                   (poly-set 'and
                             nil
                             (add-linear-terms
                              :lhs rhs
                              :rhs lhs
-                             (base-poly ttree
-                                        '<=
-                                        (and (ts-rationalp ts-lhs)
-                                             (ts-rationalp ts-rhs))
-                                        nil))))))))
+                             (let ((rationalp-flg
+                                    (and (ts-rationalp ts-lhs)
+                                         (ts-rationalp ts-rhs))))
+                               (base-poly0 (if rationalp-flg ts-ttree ttree)
+
+; See the "break from tradition" comment above for a discussion of the
+; parents.
+
+                                           (collect-parents ttree)
+                                           '<=
+                                           rationalp-flg
+                                           nil)))))))))
          ((equalityp atm)
           (let ((lhs (fargn atm 1))
                 (rhs (fargn atm 2)))
@@ -494,100 +528,122 @@
 ; is 'T.
 
                 (if positivep
-                    ;; (implies (equal lhs rhs) 
-                    ;;          (and (<= lhs rhs) (<= rhs lhs)))
-                    (poly-set 'and
-                              (add-linear-terms :lhs lhs
-                                                :rhs rhs
-                                                (base-poly ttree
-                                                           '<=
-                                                           (and (ts-rationalp
-                                                                 ts-lhs)
-                                                                (ts-rationalp
-                                                                 ts-rhs))
-                                                           t))
-                              (add-linear-terms :lhs rhs
-                                                :rhs lhs
-                                                (base-poly ttree
-                                                           '<=
-                                                           (and (ts-rationalp
-                                                                 ts-lhs)
-                                                                (ts-rationalp
-                                                                 ts-rhs))
-                                                           t)))
-                  ;; (not (equal lhs rhs))
+
+; (implies (equal lhs rhs) 
+;          (and (<= lhs rhs) (<= rhs lhs)))
+
+                    (let ((rationalp-flg (and (ts-rationalp ts-lhs)
+                                              (ts-rationalp ts-rhs))))
+                      (poly-set 'and
+                                (add-linear-terms
+                                 :lhs lhs
+                                 :rhs rhs
+                                 (base-poly0 (if rationalp-flg ts-ttree ttree)
+
+; See the "break from tradition" comment above for a discussion of the
+; parents.
+
+                                             (collect-parents ttree)
+                                             '<=
+                                             rationalp-flg
+                                             t))
+                                (add-linear-terms
+                                 :lhs rhs
+                                 :rhs lhs
+                                 (base-poly0 (if rationalp-flg ts-ttree ttree)
+
+; See the "break from tradition" comment above for a discussion of the
+; parents.
+
+                                             (collect-parents ttree)
+                                             '<=
+                                             rationalp-flg
+                                             t))))
+
+; Other case: (not (equal lhs rhs)).  But we need additional (type) information
+; in order to derive inequalities.
+
                   (cond ((and (ts-subsetp ts-lhs *ts-integer*)
                               (ts-subsetp ts-rhs *ts-integer*))
-                         ;; (implies (and (not (equal lhs rhs))
-                         ;;               (integerp lhs)
-                         ;;               (integerp rhs))
-                         ;;          (or (<= (1+ lhs) rhs)
-                         ;;              (<= (1+ rhs) lhs)))
+
+; (implies (and (not (equal lhs rhs))
+;               (integerp lhs)
+;               (integerp rhs))
+;          (or (<= (1+ lhs) rhs)
+;              (<= (1+ rhs) lhs)))
+
                          (poly-set 'or
-                                   (add-linear-terms :lhs lhs
-                                                     :lhs *1*
-                                                     :rhs rhs
-                                                     (base-poly ts-ttree
-                                                                '<=
-                                                                t
-                                                                nil))
-                                   (add-linear-terms :lhs rhs
-                                                     :lhs *1*
-                                                     :rhs lhs
-                                                     (base-poly ts-ttree
-                                                                '<=
-                                                                t
-                                                                nil))))
+                                   (add-linear-terms
+                                    :lhs lhs
+                                    :lhs *1*
+                                    :rhs rhs
+                                    (base-poly ts-ttree
+                                               '<=
+                                               t
+                                               nil))
+                                   (add-linear-terms
+                                    :lhs rhs
+                                    :lhs *1*
+                                    :rhs lhs
+                                    (base-poly ts-ttree
+                                               '<=
+                                               t
+                                               nil))))
                         ((if (ts-subsetp ts-lhs *ts-acl2-number*)
                              (or (ts-subsetp ts-rhs *ts-acl2-number*)
                                  (not (ts-intersectp ts-lhs *ts-zero*)))
                            (and (ts-subsetp ts-rhs *ts-acl2-number*)
                                 (not (ts-intersectp ts-rhs *ts-zero*))))
-                         ;; (implies (and (not (equal lhs rhs))
-                         ;;               (or (and (acl2-numberp lhs)
-                         ;;                        (acl2-numberp rhs))
-                         ;;                   (and (acl2-numberp lhs)
-                         ;;                        (not (equal lhs 0)))
-                         ;;                   (and (acl2-numberp rhs)
-                         ;;                        (not (equal rhs 0)))))
-                         ;;          (or (< lhs rhs)
-                         ;;              (< rhs lhs)))
-                         (poly-set 'or
-                                   (add-linear-terms
-                                    :lhs lhs
-                                    :rhs rhs
-                                    (base-poly ts-ttree
-                                               '<
-                                               (and (ts-rationalp ts-lhs)
-                                                    (ts-rationalp ts-rhs))
-                                               nil))
-                                   (add-linear-terms
-                                    :lhs rhs
-                                    :rhs lhs
-                                    (base-poly ts-ttree
-                                               '<
-                                               (and (ts-rationalp ts-lhs)
-                                                    (ts-rationalp ts-rhs))
-                                               nil))))
+
+; (implies (and (not (equal lhs rhs))
+;               (or (and (acl2-numberp lhs)
+;                        (acl2-numberp rhs))
+;                   (and (acl2-numberp lhs)
+;                        (not (equal lhs 0)))
+;                   (and (acl2-numberp rhs)
+;                        (not (equal rhs 0)))))
+;          (or (< lhs rhs)
+;              (< rhs lhs)))
+
+                         (let ((rationalp-flg (and (ts-rationalp ts-lhs)
+                                                   (ts-rationalp ts-rhs))))
+                           (poly-set 'or
+                                     (add-linear-terms
+                                      :lhs lhs
+                                      :rhs rhs
+                                      (base-poly ts-ttree
+                                                 '<
+                                                 rationalp-flg
+                                                 nil))
+                                     (add-linear-terms
+                                      :lhs rhs
+                                      :rhs lhs
+                                      (base-poly ts-ttree
+                                                 '<
+                                                 rationalp-flg
+                                                 nil)))))
                         ((and (ts-acl2-numberp ts-lhs)
                               force-flg
                               (ts-intersectp ts-rhs *ts-acl2-number*))
-                         ;; (implies (and (not (equal lhs rhs))
-                         ;;               (acl2-numberp lhs)
-                         ;;               (force (acl2-numberp rhs)))
-                         ;;          (or (< lhs rhs)
-                         ;;              (< rhs lhs)))
+
+; (implies (and (not (equal lhs rhs))
+;               (acl2-numberp lhs)
+;               (force (acl2-numberp rhs)))
+;          (or (< lhs rhs)
+;              (< rhs lhs)))
+
                          (mv-let (flg new-ttree)
                                  (add-linear-assumption
                                   term
                                   `(acl2-numberp ,rhs)
                                   type-alist ens
                                   (immediate-forcep nil ens)
-                                  force-flg wrld ttree)
+                                  force-flg wrld ts-ttree)
 
 ; We strongly suspect that add-linear-assumption will succeed with flg =
 ; :added, since (ts-intersectp ts-rhs *ts-acl2-number*).  But we do not depend
 ; on this without checking it.
+
                                  (assert$
                                   (and (not (eq flg :failed))
                                        (not (eq flg :known-false)))
@@ -609,18 +665,20 @@
                         ((and (ts-acl2-numberp ts-rhs)
                               force-flg
                               (ts-intersectp ts-lhs *ts-acl2-number*))
-                         ;; (implies (and (not (equal lhs rhs))
-                         ;;               (acl2-numberp rhs)
-                         ;;               (force (acl2-numberp lhs)))
-                         ;;          (or (< lhs rhs)
-                         ;;              (< rhs lhs)))
+
+; (implies (and (not (equal lhs rhs))
+;               (acl2-numberp rhs)
+;               (force (acl2-numberp lhs)))
+;          (or (< lhs rhs)
+;              (< rhs lhs)))
+
                          (mv-let (flg new-ttree)
                                  (add-linear-assumption
                                   term
                                   `(acl2-numberp ,lhs)
                                   type-alist ens
                                   (immediate-forcep nil ens)
-                                  force-flg wrld ttree)
+                                  force-flg wrld ts-ttree)
                                  (assert$ ; see comment on preceding assert$
                                   (and (not (eq flg :failed))
                                        (not (eq flg :known-false)))
@@ -641,7 +699,6 @@
                                                         nil))))))
                         (t
                          nil)))))))
-
        ((quotep atm)
 
 ; This is a strange one.  It can happen that the
@@ -711,7 +768,6 @@
                         nil
                         (impossible-poly ttree))
               nil)))
-
        (t nil))))))
 
 (defun linearize (term positivep type-alist ens force-flg wrld ttree state)
@@ -719,12 +775,12 @@
 ; If positivep we are to linearize term; else we are to linearize the negation
 ; of term.  The linearization of a term is either NIL, meaning no linear
 ; information was extracted from the term; or else it is a list of length 1
-; containing a list of polynomials ((p1...pn)) each of which is implied by the
-; term linearized; or else it is a list of length 2, ((p1...pn) (q1...qn)),
-; each element of which is a list of polynomials such that either all of the ps
-; or all of qs are implied by the term linearized.  The claim that the polys
-; are implied by the term has to be taken with a grain of salt: the additional
-; 'assumptions in the ttree fields of the polys must be considered.
+; containing a list of polynomials ((p1...pn)) such that term implies their
+; conjunction (p1&...&pn); or else it is a list of length 2, ((p1...pn)
+; (q1...qn)), such that term implies the disjunction (p1&...&pn) \/
+; (q1&...&qn).  The claim that the term implies the polys has to be taken with
+; a grain of salt: the additional 'assumptions in the ttree fields of the polys
+; must be considered.
 
 ; There are two situations where this code might add an assumption to the polys
 ; it creates.  The first is that we sometimes call type-set and may get back a
