@@ -125,6 +125,7 @@
               (getprop name 'formals nil 'current-acl2-world
                        (f-get-global 'current-acl2-world *the-live-state*))))))
 
+#-acl2-mv-as-values
 (defun trace-values (name)
   (list 'cons
         '(car values)
@@ -132,6 +133,11 @@
           (cond ((eql mul 1) nil)
                 (t (list 'acl2::mv-refs (1- mul)))))))
 
+#+acl2-mv-as-values
+(defun trace-values (name)
+  (declare (ignore name))
+  'values)
+
 ; We construct the (ccl:advise <fn-name> ... :when :after) form that performs the
 ; tracing on exit.
 
