@@ -1,4 +1,4 @@
-; ACL2 Version 3.0 -- A Computational Logic for Applicative Common Lisp
+; ACL2 Version 3.1 -- A Computational Logic for Applicative Common Lisp
 ; Copyright (C) 2006  University of Texas at Austin
 
 ; This version of ACL2 is a descendent of ACL2 Version 1.9, Copyright
@@ -184,7 +184,10 @@
 
 (defun intern-in-keyword-package (sym)
   (declare (xargs :guard (symbolp sym)))
-  (intern (symbol-name sym) "KEYWORD"))
+  (let ((ans
+; See comment in intern-in-package-of-symbol for an explanation of this trick.
+         (intern (symbol-name sym) "KEYWORD")))
+    ans))
 
 (defun make-pretty-pc-command (x)
   (declare (xargs :guard (symbolp x)))
@@ -793,9 +796,9 @@
         (cdr goals)
         goals)))
 
-(defun make-pc-ens (pc-ens w)
+(defun make-pc-ens (pc-ens state)
   (if (null pc-ens)
-      (global-val 'global-enabled-structure w)
+      (ens state)
     pc-ens))
 
 (defun initial-rcnst-from-ens (ens wrld)
@@ -849,7 +852,7 @@
                     (list (cons #\0 instr)))
                    (mv 'pc-single-step-error-primitive nil state)))
           ((car vals) ;so, there is a new state
-           (let ((pc-ens (make-pc-ens (pc-ens) wrld)))
+           (let ((pc-ens (make-pc-ens (pc-ens) state)))
              (mv-let
               (bad-ass ttree)
               (resume-suspended-assumption-rewriting
