@@ -1,4 +1,4 @@
-; ACL2 Version 3.0 -- A Computational Logic for Applicative Common Lisp
+; ACL2 Version 3.1 -- A Computational Logic for Applicative Common Lisp
 ; Copyright (C) 2006  University of Texas at Austin
 
 ; This version of ACL2 is a descendent of ACL2 Version 1.9, Copyright
@@ -449,8 +449,15 @@
   (list 'the-type-set (ts-union-fn x)))
 
 (defmacro ts-subsetp (ts1 ts2)
-  (list 'let (list (list 'ts1 ts1)
-                   (list 'ts2 ts2))
+  (list 'let
+        (list (list 'ts1 ts1)
+              (list 'ts2 ts2))
+
+; Warning: Keep the following type in sync with the definition of the-type-set
+; in def-basic-type-sets.
+
+        `(declare (type (integer ,*min-type-set* ,*max-type-set*)
+                        ts1 ts2))
         '(ts= (ts-intersection ts1 ts2) ts1)))
 
 ;; RAG - I modified this to include cases for the irrationals and
